// AnnotationDialog.cpp : implementation file
//

#include "stdafx.h"
#include "GV.h"
#include "AnnotationDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAnnotationDialog dialog


CAnnotationDialog::CAnnotationDialog(CView* pParent)
	: CDialog(CAnnotationDialog::IDD, pParent)
{
	m_pView = pParent;

	//{{AFX_DATA_INIT(CAnnotationDialog)
	m_accession = _T("");
	m_ec = _T("");
	m_locus = _T("");
	m_locus_tag = _T("");
	m_note = _T("");
	m_range = _T("");
	m_strand = -1;
	m_type = -1;
	m_gi = _T("");
	m_product = _T("");
	//}}AFX_DATA_INIT
}


void CAnnotationDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAnnotationDialog)
	DDX_Text(pDX, IDC_ACCESSION_EDIT, m_accession);
	DDX_Text(pDX, IDC_EC_EDIT, m_ec);
	DDX_Text(pDX, IDC_LOCUS_EDIT, m_locus);
	DDX_Text(pDX, IDC_LOCUS_TAG_EDIT, m_locus_tag);
	DDX_Text(pDX, IDC_NOTE_EDIT, m_note);
	DDX_Text(pDX, IDC_RANGE_EDIT, m_range);
	DDX_CBIndex(pDX, IDC_STRAND_COMBO, m_strand);
	DDX_CBIndex(pDX, IDC_TYPE_COMBO, m_type);
	DDX_Text(pDX, IDC_GI_EDIT, m_gi);
	DDX_Text(pDX, IDC_PRODUCT_EDIT, m_product);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CAnnotationDialog, CDialog)
	//{{AFX_MSG_MAP(CAnnotationDialog)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAnnotationDialog message handlers
// CPrimerDialog message handlers
BOOL CAnnotationDialog::Create() 
{
	return CDialog::Create(CAnnotationDialog::IDD);
}

BOOL CAnnotationDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CAnnotationDialog::OnOK() 
{
	UpdateData(TRUE);

	try{

		// Save the users choices
		edit_annot.range( trim_space( LPCSTR(m_range) ) );

		edit_annot.type(m_type);
		edit_annot.strand( (m_strand == 0) ? Seq_strand_plus : Seq_strand_minus );
		edit_annot.info(GeneAnnotation::LOCUS, trim_space( LPCSTR(m_locus) ) );
		edit_annot.info(GeneAnnotation::LOCUS_TAG, trim_space( LPCSTR(m_locus_tag) ) );
		edit_annot.info(GeneAnnotation::PRODUCT, trim_space( LPCSTR(m_product) ) );

		// Use the pack_string function to remove all end of line and carrige return characters
		edit_annot.info(GeneAnnotation::NOTE, pack_string( trim_space( LPCSTR(m_note) ) ) );
		edit_annot.info(GeneAnnotation::EC, trim_space( LPCSTR(m_ec) ) );
		
		m_gi = trim_space( LPCSTR(m_gi) ).c_str();
		m_accession = trim_space( LPCSTR(m_accession) ).c_str();

		int gi = -1;
			
		if( !m_gi.IsEmpty() ){
			
			if(integer( LPCSTR(m_gi) ) == false){
				throw "gi must be a positive  integer";
			}
				
			gi = atoi( LPCSTR(m_gi) );
		}

		SeqIdPtr sip = NULL;
			
		if(gi > 0){
			
			sip = ValNodeNew(NULL);
			sip->choice = SEQID_GI;
			sip->data.intvalue = gi;
		}
			
		if( !m_accession.IsEmpty() ){
			
			if(sip == NULL){
				
				if(m_accession.Find('|') != -1){
					// Yes
					sip = SeqIdParse ( (char*)LPCSTR(m_accession) );
				}
				else{
					// No
					sip = SeqIdParse( (char*)LPCSTR( CString("gb|" + m_accession) ) );
				}
				
				if(sip == NULL){
					throw "Unable to write accession";
				}
			}
			else{
				if(m_accession.Find('|') != -1){
					// Yes
					sip->next = SeqIdParse ( (char*)LPCSTR(m_accession) );
				}
				else{
					// No
					sip->next = SeqIdParse( (char*)LPCSTR( CString("gb|" + m_accession) ) );
				}
				
				if(sip->next == NULL){
					throw "Unable to write accession";
				}
			}
		}
			
		if(sip){
			// Only update the seqid if the user has provided the info
			edit_annot.seqid(sip);
			sip = SeqIdSetFree(sip);
		}

		if(m_pView != NULL){
			m_pView->PostMessage(WM_ANNOTATION_APPLY, 0);
		}

	}
	catch(const char *error){

		AfxMessageBox(error);
	}
	catch(...){
		AfxMessageBox("Unable to apply -- please check for typos");
	}
}

void CAnnotationDialog::OnCancel() 
{
	if(m_pView != NULL){
		m_pView->PostMessage(WM_ANNOTATION_CLOSE, 0);
	}
}

void CAnnotationDialog::refresh_dialog(GenomePage &m_gp)
{
	if(m_pView == NULL){
		return;
	}

	int gi = -1;

	// Only update if the user has selected a single annotation
	edit_annot = m_gp.selected_annotation();
		
	if( edit_annot.is_genome_valid() == false){
	
		AfxMessageBox("Please select a genome before editing annotations");
		return;
	}

	if( edit_annot.is_annot_valid() == false){
		
		// Reset all defaults
		m_range = "";
		m_locus = "";
		m_locus_tag = "";
		m_product = "";
		m_note = "";
		m_ec = "";
		m_gi = "";
		m_accession = "";
		m_strand = Seq_strand_plus;
		m_type = GeneAnnotation::CDS;
	}
	else{

		m_range = edit_annot.range().c_str();
		m_locus = edit_annot.info(GeneAnnotation::LOCUS).c_str();
		m_locus_tag = edit_annot.info(GeneAnnotation::LOCUS_TAG).c_str();
		m_product = edit_annot.info(GeneAnnotation::PRODUCT).c_str();
		
		// Use a maximum line length of 50 characters
		m_note = wrap_string( edit_annot.info(GeneAnnotation::NOTE), 50).c_str();
		
		m_ec = edit_annot.info(GeneAnnotation::EC).c_str();
		gi = edit_annot.gi();
		m_accession = edit_annot.accession(true).c_str();
		m_type = edit_annot.type();
		m_strand = (edit_annot.strand() == Seq_strand_plus) ? 0 : 1;
	}

	UpdateData(FALSE);
}
