// This program was prepared by the Regents of the University of California at 
// Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36 
// with the U.S. Department of Energy (DOE). All rights in the program are reserved 
// by the DOE and the University.  Permission is granted to the public to copy and 
// use this software without charge, provided that this Notice and any statement of 
// authorship are reproduced on all copies.  Neither the U.S. Government nor the 
// University makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this software.

// DownLoadDialog.cpp : implementation file
//

#include "stdafx.h"
#include "GV.h"
#include "DownLoadDialog.h"
#include "ncbi_util.h"
#include <sstream>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDownLoadDialog dialog


CDownLoadDialog::CDownLoadDialog()
	: CDialog(CDownLoadDialog::IDD, NULL)
{
	//{{AFX_DATA_INIT(CDownLoadDialog)
	m_accession = _T("");
	m_choice = 0;
	//}}AFX_DATA_INIT

	save_file = false;
}


void CDownLoadDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDownLoadDialog)
	DDX_Control(pDX, IDC_RESULTS_LIST, m_results);
	DDX_Text(pDX, IDC_EDIT_ACCESION, m_accession);
	DDX_CBIndex(pDX, IDC_DOWNLOAD_COMBO, m_choice);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDownLoadDialog, CDialog)
	//{{AFX_MSG_MAP(CDownLoadDialog)
	ON_BN_CLICKED(IDC_SEARCH_BUTTON, OnSearchButton)
	ON_BN_CLICKED(ID_LOAD_SAVE, OnLoadSave)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDownLoadDialog message handlers


void CDownLoadDialog::OnSearchButton() 
{
	// Clear any previous results
	m_results.ResetContent();
	docsum_list.clear();

	UpdateData(TRUE);

	try{
		
		// What search string has the user entered?
		switch(m_choice){
			case 0:
				docsum_list = docsum_list_from_search_key(LPCSTR(m_accession), 
					ENTREZ_GENOME_ACCESSION);
				break;
			case 1:
				docsum_list = docsum_list_from_search_key(LPCSTR(m_accession), 
					ENTREZ_GENOME_GI);
				break;
			case 2:
				docsum_list = docsum_list_from_search_key(LPCSTR(m_accession), 
					ENTREZ_GENOME_TAXA_NAME);
				break;
			case 3:
				docsum_list = docsum_list_from_search_key(LPCSTR(m_accession), 
					ENTREZ_GENOME_USER_DEFINED);
				break;			
			default:
				throw "Unknown download search key!";
		};
	}
	catch(const char *error){
		m_results.AddString(error);
		return;
	}
	catch(...){
		m_results.AddString("Error downloading data");
		return;
	}

	if(docsum_list.size() == 0){
		m_results.AddString("No entries found");
		return;
	}

	// Populate the list box with Document Summaries (DocSums)
	std::list< std::pair<unsigned int, std::string> >::iterator iter;
	unsigned int max_string_length = 0;
	unsigned int list_count;
	const unsigned int MAX_LIST_BOX_ENTRIES = 3600;
	CDC *pDC;

	for(list_count = 0, iter = docsum_list.begin();iter != docsum_list.end();iter++, list_count++){
		std::stringstream s;

		// Explain to users that they've encountered a limitation of the 
		// MFC CListBox class!
		if(list_count > MAX_LIST_BOX_ENTRIES){
			AfxMessageBox("The maximum number of entries that can be displayed in a\r\n"
				"ListBox has been reached - list has been truncated!");
			break;
		}

		s << "(" << iter->first << ") " << iter->second;
		
		unsigned int str_len = s.str().size();

		pDC = m_results.GetDC();

		if(pDC->GetTextExtent(s.str().c_str(), str_len).cx > max_string_length){
			max_string_length = pDC->GetTextExtent(s.str().c_str(), str_len).cx;
		}
			
		m_results.ReleaseDC(pDC);

		m_results.AddString(s.str().c_str());
	}

	// Set the width of the list box for proper horizontal
	// scrolling (a kludge to get around horizontal scroll
	// limitations of the CListBox).
	m_results.SetHorizontalExtent(max_string_length);
}

void CDownLoadDialog::OnOK() 
{
	// Get the entires selected by the user
	std::list< std::pair<unsigned int, std::string> >::iterator iter;
	unsigned int i;

	for(i = 0, iter = docsum_list.begin();iter != docsum_list.end();iter++, i++){
		if(m_results.GetSel(i)){
			gi_list.push_back(iter->first);
		}
	}


	CDialog::OnOK();
}

void CDownLoadDialog::OnLoadSave() 
{
	std::list< std::pair<unsigned int, std::string> >::iterator iter;
	unsigned int i;

	for(i = 0, iter = docsum_list.begin();iter != docsum_list.end();iter++, i++){
		if(m_results.GetSel(i)){
			gi_list.push_back(iter->first);
		}
	}

	// We can only save one entry to disk
	if(gi_list.size() != 1){
		gi_list.clear();

		AfxMessageBox("Please select one and only one entry to save");
		
		return;
	}

	// Prompt the user for a filename
	CFileDialog fDlg(FALSE); // Save as ...

	fDlg.m_ofn.lpstrTitle = "Save ASCII annotation (asn) file";
	fDlg.m_ofn.lpstrFilter = "ASN.1\0*.asn\0All Files\0*.*\0\0";
	
	if(fDlg.DoModal() == IDOK){
		file_name = LPCSTR(fDlg.GetFileName());
		save_file = true;

		CDialog::OnOK();
	}
}
