// This program was prepared by the Regents of the University of California at 
// Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36 
// with the U.S. Department of Energy (DOE). All rights in the program are reserved 
// by the DOE and the University.  Permission is granted to the public to copy and 
// use this software without charge, provided that this Notice and any statement of 
// authorship are reproduced on all copies.  Neither the U.S. Government nor the 
// University makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this software.

// GV.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "GV.h"

#include "MainFrm.h"
#include "GVDoc.h"
#include "GVView.h"
#include "GVCommandLine.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGVApp

BEGIN_MESSAGE_MAP(CGVApp, CWinApp)
	//{{AFX_MSG_MAP(CGVApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	//ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	// Standard print setup command
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGVApp construction

CGVApp::CGVApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CGVApp object

CGVApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CGVApp initialization

BOOL CGVApp::InitInstance()
{
	if ( !AfxSocketInit() )
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}

	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// I have chosen "LANL B" as the key -- is there a standard LANL key
	// out there?
	SetRegistryKey(_T("LANL B"));

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)
	
	// Initialize the Rich text control in the search dialog
	AfxInitRichEdit();

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.

	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CGVDoc),
		RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		RUNTIME_CLASS(CGVView));
	AddDocTemplate(pDocTemplate);

	// Parse command line for standard shell commands, DDE, file open
	//CCommandLineInfo cmdInfo;
	//ParseCommandLine(cmdInfo);

	// Parse command line for standard shell commands, DDE, file open
	CGVCommandLine cmdInfo;

	try{
		ParseCommandLine(cmdInfo);
	}
	catch(const char *error){
		AfxMessageBox(error);
		
		// We caught an error. Clear the cmdInfo structure to make
		// sure that we don't try an open a non-existant file.
		cmdInfo.clear();
	}

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo)){
		return FALSE;
	}

	// Do we need to process any input data? Note that we must wait until
	// after ProcessShellCommand (for the GenomePage data structure to 
	// be initialized).
	if(cmdInfo.m_import_filename.empty() == false){
		POSITION doc_loc = pDocTemplate->GetFirstDocPosition();
		CGVDoc *pDoc = (CGVDoc *)(pDocTemplate->GetNextDoc(doc_loc));

		pDoc->gp.add_files_to_open(cmdInfo.m_import_filename);

		try{
			// Process any files that were specified on the command line
			pDoc->gp.process_files_to_open();
		}
		catch(const char *error){
			AfxMessageBox(error);
		}

	}

	// The one and only window has been initialized, so show and update it.
	m_pMainWnd->ShowWindow(SW_SHOW);
	m_pMainWnd->UpdateWindow();

	return TRUE;
}


/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CString	m_info;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
		// No message handlers
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	m_info = _T("");
	//}}AFX_DATA_INIT

	// Give credit where credit is due ...
	m_info.Format(
		"This program was prepared by the Regents of the University of California\r\n"
		"at Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36\r\n"
		"with the U.S. Department of Energy (DOE). All rights in the program are reserved by\r\n"
		"the DOE and the University.  Permission is granted to the public to copy and use this\r\n"
		"software without charge, provided that this Notice and any statement of authorship are\r\n"
		"reproduced on all copies.  Neither the U.S. Government nor the University makes any\r\n"
		"warranty, express or implied, or assumes any liability or responsibility for the use of\r\n"
		"this software.\r\n"
		"\r\n"
		"Copyright (C) 2004\r\n"
		"J.D. Gans\r\n"
		"jgans@lanl.gov\r\n"
		"Los Alamos National Lab\r\n"
		"Bioscience Division, B-5\r\n"
		"\r\n"
		"NCBI toolkit (ASN.1 compatibility)\r\n"
		"http://www.ncbi.nih.gov/\r\n"
		"\r\n"
		"PostScript rendering based on the work of Mark Kilgard.\r\n"
		);
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Text(pDX, IDC_CREDIT_EDIT, m_info);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

// App command to run the dialog
void CGVApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CGVApp message handlers

