// This program was prepared by the Regents of the University of California at 
// Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36 
// with the U.S. Department of Energy (DOE). All rights in the program are reserved 
// by the DOE and the University.  Permission is granted to the public to copy and 
// use this software without charge, provided that this Notice and any statement of 
// authorship are reproduced on all copies.  Neither the U.S. Government nor the 
// University makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this software.
//

#include "stdafx.h"
#include "GVCommandLine.h"
#include "io.h"

using namespace std;

// Helper function(s)
string GetWildcardPathInfo(const string &m_full_path);

void CGVCommandLine::ParseParam(LPCTSTR lpszParam, BOOL bFlag, BOOL bLast)
{
	// Have we read a flag?
	if(bFlag){
		// Do nothing with flags (for now)
		return;
	}
				
	// Does this name contain a wild card?
	if(strchr(lpszParam, '*') != NULL){
		// This string contains a wild card, save the file name(s).
		struct _finddata_t fileinfo;
		long fileHandle;

		fileHandle = _findfirst(lpszParam, &fileinfo);

		if(fileHandle == -1){
			throw "The commandline wildcard failed to match a file name";
		}
					
		string pathInfo = GetWildcardPathInfo(lpszParam);

		// Iterate over all file names that match the wildcard pattern while
		// making sure to append any path information.
		while(_findnext(fileHandle, &fileinfo) == 0){
			m_import_filename.push_back(pathInfo + fileinfo.name);
		}

		_findclose(fileHandle);
	}
	else{
		// Save the file name
		m_import_filename.push_back(lpszParam);
	}
}

string GetWildcardPathInfo(const string &m_full_path)
{
	unsigned int pos = m_full_path.find('\\');

	if(pos == string::npos){
		return "";
	}
	
	// Now search backwards for the location of the first '\' character
	while(pos >= 0){
		if(m_full_path[pos] == '\\'){
			return m_full_path.substr(0, pos + 1);
		}

		pos --;
	}
	
	// No path info to return
	return "";
}
