// This program was prepared by the Regents of the University of California at 
// Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36 
// with the U.S. Department of Energy (DOE). All rights in the program are reserved 
// by the DOE and the University.  Permission is granted to the public to copy and 
// use this software without charge, provided that this Notice and any statement of 
// authorship are reproduced on all copies.  Neither the U.S. Government nor the 
// University makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this software.

// GVView.cpp : implementation of the CGVView class
//

#include "stdafx.h"
#include "GV.h"
#include "MainFrm.h"
#include "GVDoc.h"
#include "GVView.h"
#include "DownLoadDialog.h"
#include "DialogExport.h"
#include "univout.h"
#include "primer.h"

#include <sstream>

// Include cderr.h to get FNERR_BUFFERTOOSMALL
#include "cderr.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGVView

IMPLEMENT_DYNCREATE(CGVView, CView)

BEGIN_MESSAGE_MAP(CGVView, CView)
	//{{AFX_MSG_MAP(CGVView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_SIZE()
	ON_WM_ERASEBKGND()
	ON_COMMAND(ID_FILE_OPEN, OnFileOpen)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MBUTTONDOWN()
	ON_WM_MBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_MOUSEWHEEL()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_COMMAND(ID_EDIT_CLEAR, OnEditClear)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CLEAR, OnUpdateEditClear)
	ON_COMMAND(ID_EDIT_MOVE_DOWN, OnEditMoveDown)
	ON_UPDATE_COMMAND_UI(ID_EDIT_MOVE_DOWN, OnUpdateEditMoveDown)
	ON_COMMAND(ID_EDIT_MOVE_UP, OnEditMoveUp)
	ON_UPDATE_COMMAND_UI(ID_EDIT_MOVE_UP, OnUpdateEditMoveUp)
	ON_COMMAND(ID_EDIT_COMPLEMENT, OnEditComplement)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COMPLEMENT, OnUpdateEditComplement)
	ON_COMMAND(ID_FILE_MRU_FILE1, OnFileMruFile1)
	ON_COMMAND(ID_FILE_MRU_FILE2, OnFileMruFile2)
	ON_COMMAND(ID_FILE_MRU_FILE3, OnFileMruFile3)
	ON_COMMAND(ID_FILE_MRU_FILE4, OnFileMruFile4)
	ON_WM_KEYDOWN()
	ON_COMMAND(ID_EDIT_SEARCH, OnEditSearch)
	ON_COMMAND(ID_EDIT_SEARCH_AGAIN, OnEditSearchAgain)
	ON_COMMAND(ID_FILE_EXPORT, OnFileExport)
	ON_WM_LBUTTONDBLCLK()
	ON_COMMAND(ID_EDIT_SELECT_ALL, OnEditSelectAll)
	ON_UPDATE_COMMAND_UI(ID_EDIT_SELECT_ALL, OnUpdateEditSelectAll)
	ON_COMMAND(ID_EDIT_GC, OnEditGc)
	ON_UPDATE_COMMAND_UI(ID_EDIT_GC, OnUpdateEditGc)
	ON_COMMAND(ID_VIEW_HIDE_SELECTION, OnViewHideSelection)
	ON_UPDATE_COMMAND_UI(ID_VIEW_HIDE_SELECTION, OnUpdateViewHideSelection)
	ON_COMMAND(ID_VIEW_SHOW_ALL, OnViewShowAll)
	ON_COMMAND(ID_COLOR_BACKGROUND, OnColorBackground)
	ON_COMMAND(ID_COLOR_ANNOT, OnColorAnnot)
	ON_UPDATE_COMMAND_UI(ID_COLOR_ANNOT, OnUpdateColorAnnot)
	ON_COMMAND(ID_COLOR_CDS, OnColorCds)
	ON_UPDATE_COMMAND_UI(ID_COLOR_CDS, OnUpdateColorCds)
	ON_COMMAND(ID_COLOR_GENE, OnColorGene)
	ON_UPDATE_COMMAND_UI(ID_COLOR_GENE, OnUpdateColorGene)
	ON_COMMAND(ID_COLOR_INTERGENIC, OnColorIntergenic)
	ON_UPDATE_COMMAND_UI(ID_COLOR_INTERGENIC, OnUpdateColorIntergenic)
	ON_COMMAND(ID_COLOR_RNA, OnColorRna)
	ON_UPDATE_COMMAND_UI(ID_COLOR_RNA, OnUpdateColorRna)
	ON_COMMAND(ID_COLOR_SELECT, OnColorSelect)
	ON_UPDATE_COMMAND_UI(ID_COLOR_SELECT, OnUpdateColorSelect)
	ON_COMMAND(ID_COLOR_TEXT, OnColorText)
	ON_UPDATE_COMMAND_UI(ID_COLOR_TEXT, OnUpdateColorText)
	ON_COMMAND(ID_FILE_NCBI_ASN, OnFileNcbiAsn)
	ON_COMMAND(ID_EDIT_TM, OnEditTm)
	ON_UPDATE_COMMAND_UI(ID_EDIT_TM, OnUpdateEditTm)
	ON_COMMAND(ID_VIEW_SHOW_LESS, OnViewShowLess)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SHOW_LESS, OnUpdateViewShowLess)
	ON_COMMAND(ID_VIEW_SHOW_MORE, OnViewShowMore)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SHOW_MORE, OnUpdateViewShowMore)
	ON_WM_CHAR()
	ON_COMMAND(ID_SEQUENCE_SHOW_AA, OnSequenceShowAa)
	ON_UPDATE_COMMAND_UI(ID_SEQUENCE_SHOW_AA, OnUpdateSequenceShowAa)
	ON_COMMAND(ID_SEQUENCE_SHOW_NA, OnSequenceShowNa)
	ON_UPDATE_COMMAND_UI(ID_SEQUENCE_SHOW_NA, OnUpdateSequenceShowNa)
	ON_COMMAND(ID_FILE_GENOME_PLOT, OnFileGenomePlot)
	ON_COMMAND(ID_EDIT_GENOME_PLOT, OnEditGenomePlot)
	ON_UPDATE_COMMAND_UI(ID_EDIT_GENOME_PLOT, OnUpdateEditGenomePlot)
	ON_COMMAND(ID_VIEW_OPTIONS, OnViewOptions)
	ON_UPDATE_COMMAND_UI(ID_VIEW_OPTIONS, OnUpdateViewOptions)
	ON_COMMAND(ID_EDIT_HYBRID, OnEditHybrid)
	ON_COMMAND(ID_EDIT_GC_SKEW, OnEditGcSkew)
	ON_COMMAND(ID_EDIT_KETO_SKEW, OnEditKetoSkew)
	ON_COMMAND(ID_EDIT_PURINE_SKEW, OnEditPurineSkew)
	ON_COMMAND(ID_EDIT_AT, OnEditAt)
	ON_COMMAND(ID_EDIT_AT_SKEW, OnEditAtSkew)
	ON_UPDATE_COMMAND_UI(ID_EDIT_AT, OnUpdateEditAt)
	ON_UPDATE_COMMAND_UI(ID_EDIT_AT_SKEW, OnUpdateEditAtSkew)
	ON_UPDATE_COMMAND_UI(ID_EDIT_GC_SKEW, OnUpdateEditGcSkew)
	ON_UPDATE_COMMAND_UI(ID_EDIT_KETO_SKEW, OnUpdateEditKetoSkew)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PURINE_SKEW, OnUpdateEditPurineSkew)
	ON_COMMAND(ID_COLOR_CUSTOM, OnColorCustom)
	ON_UPDATE_COMMAND_UI(ID_COLOR_CUSTOM, OnUpdateColorCustom)
	ON_COMMAND(ID_COLOR_PSEUDO_GENE, OnColorPseudoGene)
	ON_UPDATE_COMMAND_UI(ID_COLOR_PSEUDO_GENE, OnUpdateColorPseudoGene)
	ON_COMMAND(ID_GENOME_PRIMER, OnGenomePrimer)
	ON_UPDATE_COMMAND_UI(ID_GENOME_PRIMER, OnUpdateGenomePrimer)
	ON_COMMAND(ID_GENOME_PRIMER_CLEAR, OnGenomePrimerClear)
	ON_UPDATE_COMMAND_UI(ID_GENOME_PRIMER_CLEAR, OnUpdateGenomePrimerClear)
	ON_COMMAND(ID_COLOR_PRIMER, OnColorPrimer)
	ON_UPDATE_COMMAND_UI(ID_COLOR_PRIMER, OnUpdateColorPrimer)
	ON_COMMAND(ID_VIEW_SIZE_CENTRIC, OnViewSizeCentric)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SIZE_CENTRIC, OnUpdateViewSizeCentric)
	ON_COMMAND(ID_VIEW_GENE_CENTRIC, OnViewGeneCentric)
	ON_UPDATE_COMMAND_UI(ID_VIEW_GENE_CENTRIC, OnUpdateViewGeneCentric)
	ON_COMMAND(ID_VIEW_HIDE_ANNOTATION, OnViewHideAnnotation)
	ON_UPDATE_COMMAND_UI(ID_VIEW_HIDE_ANNOTATION, OnUpdateViewHideAnnotation)
	ON_COMMAND(ID_VIEW_SHOW_ANNOTATION, OnViewShowAnnotation)
	ON_UPDATE_COMMAND_UI(ID_VIEW_SHOW_ANNOTATION, OnUpdateViewShowAnnotation)
	ON_COMMAND(ID_GENOME_DEL_ANNOT, OnGenomeDelAnnot)
	ON_UPDATE_COMMAND_UI(ID_GENOME_DEL_ANNOT, OnUpdateGenomeDelAnnot)
	ON_COMMAND(ID_VIEW_MOMENTUM, OnViewMomentum)
	ON_UPDATE_COMMAND_UI(ID_VIEW_MOMENTUM, OnUpdateViewMomentum)
	ON_COMMAND(ID_GENOME_PLOT_CONSERVATION, OnGenomePlotConservation)
	ON_UPDATE_COMMAND_UI(ID_GENOME_PLOT_CONSERVATION, OnUpdateGenomePlotConservation)
	ON_COMMAND(ID_EDIT_HYBRID_MOL_PCR, OnEditHybridMolPcr)
	ON_WM_HSCROLL()
	ON_COMMAND(ID_FILE_EXPORT_GENOME, OnFileExportGenome)
	ON_UPDATE_COMMAND_UI(ID_FILE_EXPORT, OnUpdateFileExport)
	ON_UPDATE_COMMAND_UI(ID_FILE_EXPORT_GENOME, OnUpdateFileExportGenome)
	ON_WM_VSCROLL()
	ON_COMMAND(ID_GENOME_EDIT_ANNOT, OnGenomeEditAnnot)
	ON_UPDATE_COMMAND_UI(ID_GENOME_EDIT_ANNOT, OnUpdateGenomeEditAnnot)
	ON_COMMAND(ID_GENOME_IMPORT_ANNOT, OnGenomeImportAnnot)
	ON_UPDATE_COMMAND_UI(ID_GENOME_IMPORT_ANNOT, OnUpdateGenomeImportAnnot)
	ON_COMMAND(ID_VIEW_RESTORE, OnViewRestore)
	ON_UPDATE_COMMAND_UI(ID_VIEW_RESTORE, OnUpdateViewRestore)
	ON_COMMAND(ID_VIEW_MAGNIFY, OnViewMagnify)
	ON_UPDATE_COMMAND_UI(ID_VIEW_MAGNIFY, OnUpdateViewMagnify)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_FORCE_UPDATE, postRedisplay)
	ON_MESSAGE(WM_SEARCH_CLOSE, OnEditSearchClose)
	ON_MESSAGE(WM_SEARCH_START, Search)
	ON_MESSAGE(WM_SEARCH_NEXT, ShowNextSearch)
	ON_MESSAGE(WM_HYBRID_NEXT, ShowNextSearch)
	ON_MESSAGE(WM_HYBRID_MOL_PCR_NEXT, ShowNextSearch)
	ON_MESSAGE(WM_HYBRID_CLOSE, OnEditHybridClose)
	ON_MESSAGE(WM_HYBRID_MOL_PCR_CLOSE, OnEditHybridMolPcrClose)
	ON_MESSAGE(WM_HYBRID_MOL_PCR_PRIMER, MOLPCRHybridPrimer)
	ON_MESSAGE(WM_HYBRID_PRIMER, HybridPrimer)
	ON_MESSAGE(WM_HYBRID_PROBE, HybridProbe)
	ON_MESSAGE(WM_HYBRID_ANNOTATE, AnnotateSelection)
	ON_MESSAGE(WM_SEARCH_ANNOTATE, AnnotateSelection)
	ON_MESSAGE(WM_HYBRID_MOL_PCR_ANNOTATE, AnnotateSelection)
	ON_MESSAGE(WM_INFO_CLOSE, OnInfoClose)
	ON_MESSAGE(WM_OPTIONS_CLOSE, OnOptionsClose)
	ON_MESSAGE(WM_OPTIONS_UPDATE, OnOptionsUpdate)
	ON_MESSAGE(WM_PRIMER_OK, PrimerOk)
	ON_MESSAGE(WM_PRIMER_CLOSE, PrimerClose)
	ON_MESSAGE(WM_ANNOTATION_APPLY, AnnotationApply)
	ON_MESSAGE(WM_ANNOTATION_CLOSE, AnnotationClose)
	// Standard printing commands
	ON_COMMAND(ID_FILE_PRINT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT, CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW, CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGVView construction/destruction

CGVView::CGVView() :
	m_hRC(0), m_pDC(0)
{
	// No font call list yet!
	font_list_id = -1;

	// 0 == high res
	// 1 == low res
	font_res = GP_POLYGON_FONT_RESOLUTION;

	printScaleMul = printScaleDiv = 82; // Empirical font scaling value

	// By default, use a 16 bit Z buffer and 16 bit color depth
	zbuf = 16;
	bpp = 16;

	// No font file unless this is set to true!
	create_font_file = false;

	// Allocate memory for the search dialog
	m_ptrSearchDlg = new CSearchDialog(this);

	// Allocate memory for the PCR/Probe search dialog
	m_ptrHybridDlg = new CHybridDialog(this);

	// Allocate memory for the MOL PCR search dialog
	m_ptrMolPCRDlg = new MolPCRHybridDialog(this);

	// Allocate memory for the info dialog
	m_ptrInfoDlg = new CInfoDialog(this);

	// Allocate memory for the options dialog
	m_ptrOptionsDlg = new COptionsDialogEx(this);

	// Allocate memory for the primer dialog
	m_ptrPrimerDlg = new CPrimerDialog(this);

	// Allocate memory for the annotation dialog
	m_ptrAnnotDlg = new CAnnotationDialog(this);
}

CGVView::~CGVView()
{
	// Remove the search dialog
	if(m_ptrSearchDlg){
		delete m_ptrSearchDlg;
	}

	// Remove the hybrid dialog
	if(m_ptrHybridDlg){
		delete m_ptrHybridDlg;
	}

	// Remove the MOL pcr dialog
	if(m_ptrMolPCRDlg){
		delete m_ptrMolPCRDlg;
	}

	// Remove the info dialog
	if(m_ptrInfoDlg){
		delete m_ptrInfoDlg;
	}

	// Remove the options dialog
	if(m_ptrOptionsDlg){
		delete m_ptrOptionsDlg;
	}

	// Remove the primer dialog
	if(m_ptrPrimerDlg){
		delete m_ptrPrimerDlg;
	}

	// Remove the annotation dialog
	if(m_ptrAnnotDlg){
		delete m_ptrAnnotDlg;
	}
}

BOOL CGVView::PreCreateWindow(CREATESTRUCT& cs)
{
	// An OpenGL window must be created with the following flags and must not
    // include CS_PARENTDC for the class style. 
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CGVView drawing

void CGVView::OnDraw(CDC* pDC)
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if(pDoc->gp.render() == true){
		// We're animating
		postRedisplay(0, 0);
	}

	if(FALSE == SwapBuffers( m_pDC->GetSafeHdc())){
		throw "SwapBuffers failed: " __FILE__;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CGVView printing

BOOL CGVView::OnPreparePrinting(CPrintInfo* pInfo)
{
	// default preparation
	return DoPreparePrinting(pInfo);
}

void CGVView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add extra initialization before printing
}

void CGVView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/)
{
	// TODO: add cleanup after printing
}

/////////////////////////////////////////////////////////////////////////////
// CGVView diagnostics

#ifdef _DEBUG
void CGVView::AssertValid() const
{
	CView::AssertValid();
}

void CGVView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGVDoc* CGVView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGVDoc)));
	return (CGVDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGVView message handlers
int CGVView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	m_pDC = new CClientDC(this);

    if(NULL == m_pDC){
		// failure to get DC
		throw "Unable to get a DC: " __FILE__;
	}

	if(!SetupPixelFormat()){
        return -1;
	}

    if(0 == (m_hRC = ::wglCreateContext(m_pDC->GetSafeHdc()))){
		throw "wglCreateContext failed: " __FILE__;
	}

    if(FALSE == ::wglMakeCurrent(m_pDC->GetSafeHdc(), m_hRC)){
		throw "wglMakeCurrent failed: " __FILE__;
	}

	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// The default font
	font.lfHeight = -pDoc->gp.font_size();
	font.lfWidth = 0;
	font.lfEscapement = 0;
	font.lfOrientation = font.lfEscapement;
	font.lfWeight = FW_BOLD /*FW_NORMAL*/;
	font.lfItalic = FALSE;
	font.lfUnderline = FALSE;
	font.lfStrikeOut = FALSE;
	font.lfCharSet = DEFAULT_CHARSET;
	font.lfOutPrecision	= OUT_DEFAULT_PRECIS;
	font.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	font.lfQuality = DEFAULT_QUALITY;
	font.lfPitchAndFamily = FF_DONTCARE|DEFAULT_PITCH;
	lstrcpy ( font.lfFaceName, "arial" );

	pDoc->gp.initialize_OGL();

	// Delete any previous font list
	if(font_list_id >= 0){
		glDeleteLists( font_list_id, GL_CHAR_COUNT );
	}

	// Generate the display list for holding the selected font
	// and set the list base to the appropriate offset.
#ifdef USE_BITMAP_FONT
	glListBase(font_list_id = GenerateBitmapListForFont(font));
#else // use a polygon font
	glListBase(font_list_id = GeneratePolygonListForFont(font));
#endif // USE_BITMAP_FONT

	// Turn on a horizontal scroll bar
	CView::EnableScrollBarCtrl(SB_HORZ);
	
	CView::SetScrollRange(SB_HORZ, HSCROLL_MIN, HSCROLL_MAX, false);
	CView::SetScrollPos(SB_HORZ, HSCROLL_DEFAULT_POS, false);

	// Turn on a vertical scroll bar
	CView::EnableScrollBarCtrl(SB_VERT);
	
	CView::SetScrollRange(SB_VERT, VSCROLL_MIN, VSCROLL_MAX, false);
	CView::SetScrollPos(SB_VERT, VSCROLL_DEFAULT_POS, false);

	return 0;
}

BOOL CGVView::SetupPixelFormat()
{
  static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
        1,                              // version number
        PFD_DRAW_TO_WINDOW |            // support window
        PFD_SUPPORT_OPENGL |			// support OpenGL
        PFD_DOUBLEBUFFER,				// double buffered
        PFD_TYPE_RGBA,                  // RGBA type
		bpp,							// bpp
        0, 0, 0, 0, 0, 0,               // color bits ignored
		0,                              // Don't use the alpha buffer
        0,                              // shift bit ignored
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits ignored
		zbuf,							// z-buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        0,                              // reserved
        0, 0, 0                         // layer masks ignored
    };

    int pixelformat;

    if(0 == (pixelformat = ::ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd))){
		throw "ChoosePixelFormat failed" __FILE__;
	}

    if(FALSE == ::SetPixelFormat(m_pDC->GetSafeHdc(), pixelformat, &pfd)){
		throw "SelectPixelFormat failed" __FILE__;
	}

	// What is the final Pixel format?
	if(DescribePixelFormat(m_pDC->GetSafeHdc(), pixelformat, 
		sizeof(PIXELFORMATDESCRIPTOR), &pfd) != 0){

		bpp = pfd.cColorBits;
		zbuf = pfd.cDepthBits;
	}
 
	
    return TRUE;
}

GLuint CGVView::GenerateBitmapListForFont(LOGFONT &font)
{
	GLuint id;

	if ( 0 == m_pDC || (GLuint)0 == (id = GetNewDisplayListIDs(GL_CHAR_COUNT)) ){
        return 0;
	}

	CFont newfont;
	CFont* oldfont;
	BOOL success;

	int old_size = font.lfHeight;

	// Scale the font as needed
	font.lfHeight = -MulDiv(-font.lfHeight, printScaleMul, printScaleDiv);

	// returns 0 if it fails
	success = newfont.CreateFontIndirect(&font);
	oldfont = m_pDC->SelectObject(&newfont);
	ASSERT(oldfont != 0);

	// Return the input font to its original state
	font.lfHeight = old_size;

    // Create a set of display lists based on the glyphs of the font 
	// notice that we really waste the first 32 spaces....
	// if there's a problem delete the display lists
	if(0 == success || 
		FALSE == ::wglUseFontBitmaps( m_pDC->m_hDC, GL_CHAR_START, GL_CHAR_COUNT, id ) ){
		::glDeleteLists( id, GL_CHAR_COUNT );
		id  = 0;
	}
	else{
		m_pDC->SelectObject( oldfont );
	}
	
	return id - GL_CHAR_START;
}

GLuint CGVView::GeneratePolygonListForFont(LOGFONT &font)
{
	GLuint id;

	if ( 0 == m_pDC || (GLuint)0 == (id = GetNewDisplayListIDs(GL_CHAR_COUNT)) ){
        return 0;
	}

	CFont newfont;
	CFont* oldfont;
	BOOL success;

	int old_size = font.lfHeight;

	// Scale the font as needed
	font.lfHeight = -MulDiv(-font.lfHeight, printScaleMul, printScaleDiv);

	// returns 0 if it fails
	success = newfont.CreateFontIndirect(&font);
	oldfont = m_pDC->SelectObject(&newfont);
	ASSERT(oldfont != 0);

	// Return the input font to its original state
	font.lfHeight = old_size;

	// Allocate structure for character metrics.
	GLYPHMETRICSFLOAT agmf[GL_CHAR_COUNT];

    // Create a set of display lists based on the glyphs of the font 
	// notice that we really waste the first 32 spaces....
	// if there's a problem delete the display lists
	if(0 == success || 
		FALSE == ::wglUseFontOutlines( m_pDC->m_hDC, 
									   GL_CHAR_START, 
									   GL_CHAR_COUNT, 
									   id,
									   // Font resolution 0 = best, 1 = worst
									   // We need to let the user set this parameter
									   // (as it effects the postscript rendering)
									   font_res, 
									   0.0, // extrution into the z direction
									   WGL_FONT_POLYGONS,
									   agmf) ){
		::glDeleteLists( id, GL_CHAR_COUNT );
		id  = 0;
	}
	else{
		m_pDC->SelectObject( oldfont );

		// Save the font metric info 
		// (this info is needed for fitting text strings into fixed size boxes).
		CGVDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);
		
		for(unsigned char i = GL_CHAR_START;i <= GL_CHAR_STOP;i++){
			pDoc->gp.set_char_dx(i, agmf[i - GL_CHAR_START].gmfCellIncX);
		}

		// A font file is a C++ source file that encodes a polygon rendered font
		// as an openGL calllist. This enables the same "look" and feel across different
		// platforms.
		if(create_font_file){
			// Set the list base in advance of write_font
			glListBase(id - GL_CHAR_START);
			write_font("c:/temp/font_file.cpp", agmf, GL_CHAR_START, GL_CHAR_STOP);
		}
	}
	
	return id - GL_CHAR_START;
}

// This is just a nice wrapper for the glGenLists() function.
GLuint CGVView::GetNewDisplayListIDs(GLsizei range)
{
	ASSERT( 0 < range );

	return ::glGenLists( range );
}

void CGVView::OnDestroy() 
{
	CView::OnDestroy();
	
	if(FALSE == ::wglDeleteContext( m_hRC ))
	{
		throw "wglDeleteContext failed: " __FILE__;
 	}

    if(m_pDC){
        delete m_pDC;
		m_pDC = NULL;
	}	
}

void CGVView::OnSize(UINT nType, int cx, int cy) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CView::OnSize(nType, cx, cy);

	if ( 0 >= cx || 0 >= cy ){
		return;
	}

	// Save the window dimensions
	viewWidth = cx;
	viewHeight = cy;

	pDoc->gp.setup_view_OGL(viewWidth, viewHeight);
}

BOOL CGVView::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE; // tell Windows not to erase the background
}

LRESULT CGVView::postRedisplay(WPARAM wParam, LPARAM lParam)
{
	InvalidateRect(0, FALSE);
	GetParent()->PostMessage(WM_PAINT);

	return 0L;
}

void CGVView::OnFileOpen() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Open a file dialog box and let the user specify an ASN.1
	// file

	CFileDialog fDlg(TRUE);

	fDlg.m_ofn.lpstrFilter = "ASN.1 files\0*.asn;*.val;*.bna\0"
							 "GenBank files\0*.gbk;*.gbff\0"
							 "EMBL files\0*.embl\0"
							 "FASTA files\0*.fna\0"
							 "All Files\0*.*\0\0";

	// Allow the user to open multiple files
	// Allow the user to select multiple files [so we must construct a 
	// string buffer to hold the returned name(s)].
	fDlg.m_ofn.nMaxFile = 2562; // <-- Windows NT 4.0 max. Dooh!

	fDlg.m_ofn.Flags = fDlg.m_ofn.Flags | OFN_ALLOWMULTISELECT;

	// Save a pointer to the buffer for easy memory clean up!
	char *buf;

	fDlg.m_ofn.lpstrFile = buf = new char [fDlg.m_ofn.nMaxFile];

	if(buf == NULL){
		AfxMessageBox("Memory allocation error!");
		return;
	}

	// Set the first character to null to prevent a default filename from 
	// appearing.
	fDlg.m_ofn.lpstrFile[0] = '\0';

	// Set the current directory
	char dirBuffer[1024];

	// If the buffer is not big enought, DON'T set the current 
	// directory
	if(GetCurrentDirectory(1023, dirBuffer) != 0){
		fDlg.m_ofn.lpstrInitialDir = dirBuffer;
	}

	if(fDlg.DoModal() == IDOK){
		if(CommDlgExtendedError() == FNERR_BUFFERTOOSMALL){
			AfxMessageBox("Too many files selected! "
				"Please select fewer files and try again.");

			delete [] buf;

			return;
		}

		POSITION pos = fDlg.GetStartPosition();
		CString file;
		int numFilesRead = 0;

		HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

		while(pos){
			numFilesRead++;
			file = fDlg.GetNextPathName(pos);

			// Open the File
			try{
				if(pDoc->gp.load(LPCTSTR(file), file_type(LPCTSTR(file))) == false){

					AfxMessageBox("Did not read any sequence data for this genome. "
						"Try downloading the file directly from NCBI");
				}
			}
			catch(const char *error){
				::SetCursor(prevCursor);

				// Was there an error opening the file?
				AfxMessageBox(error);
				
				delete [] buf;

				return;
			}
			catch(...){
				::SetCursor(prevCursor);

				// Yiikes! An unhandled error has occured!
				// Was there an error opening the file?
				AfxMessageBox("Unable to read input file!");
				
				delete [] buf;

				return;
			}
		}

		::SetCursor(prevCursor);

		if(numFilesRead == 0){
			delete [] buf;

			AfxMessageBox("Please enter a file name");
			return ;
		}

		if(numFilesRead == 1){
			// Before returning, add the file name to the MRU file list
			AfxGetApp()->AddToRecentFileList(file);
		}

		delete [] buf;

		postRedisplay(0, 0);
	}
}

void CGVView::OnFileNcbiAsn() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	
	CDownLoadDialog dlDlg;

	if(dlDlg.DoModal() == IDOK){
		HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));
		
		// Connect to the Entrez server and download the list of GI's that
		// matches the users search key
		std::list<unsigned int> gi_list;
		
		try{
			
			gi_list = dlDlg.gi_list;

			unsigned int num_gi = gi_list.size();

			if (num_gi == 0){
				throw "Please specify a valid search result to load";
			}

			std::list<unsigned int>::iterator iter;
			unsigned int i;

			bool ret = true;

			for(i = 0,iter = gi_list.begin();iter != gi_list.end();iter++, i++){
				try{


					uout << "Downloading gi " << *iter 
						 << " (" << i << " of " << num_gi << ")" << uout_endl;

					if(dlDlg.save_file){
						// Download *and* save to disk. Only allowed if the user
						// is downloading a single entry (ncbi does not have an append
						// mode to the ASN.1 api).
						if(pDoc->gp.load_and_save(*iter, dlDlg.file_name) == false){
							ret = false;
						}
					}
					else{
						if(pDoc->gp.load(*iter) == false){
							ret = false;
						}
					}
				}
				catch(...){
					// Keep any load errors silent only if
					// we're loading multiple gi's
					if(num_gi == 1){
						throw "Unable to load gi from data base";
					}
				}

				if(ret == false){
					
					AfxMessageBox("Did not read any sequence data for this genome. "
						"Try downloading the file directly from NCBI");
				}
			}

			postRedisplay(0, 0);
		}
		catch(const char *error){
			::SetCursor(prevCursor);

			// Was there an error opening the file?
			AfxMessageBox(error);

			return;
		}
		catch(...){
			::SetCursor(prevCursor);

			// Yiikes! An unhandled error has occured!
			// Was there an error opening the file?
			AfxMessageBox("Unable to download data!");

			return;
		}
	}
}


void CGVView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	// Grap the mouse
	SetCapture();

	bool update_display = false;

	if(nFlags & MK_CONTROL){
		update_display = pDoc->gp.l_button_ctrl_down(point.x, point.y);
	}
	else{
		if(nFlags & MK_SHIFT){
			std::string info;

			update_display = pDoc->gp.l_button_shift_down(point.x, point.y, info);

			if(info != ""){
				if(m_ptrInfoDlg->GetSafeHwnd() == 0){
					m_ptrInfoDlg->Create();
				}
				
				m_ptrInfoDlg->info_text(info);	
			}
		}
		else{
			update_display = pDoc->gp.l_button_down(point.x, point.y);
		}
	}

	if(update_display){
		postRedisplay(0, 0);
	}

	CView::OnLButtonDown(nFlags, point);
}

void CGVView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	// Give up control of the mouse
	ReleaseCapture();

	if(pDoc->gp.l_button_up(point.x, point.y) == true){
		postRedisplay(0, 0);
	}
	
	CView::OnLButtonUp(nFlags, point);
}

void CGVView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	std::string info;

	if(pDoc->gp.l_button_dbl_down(point.x, point.y, info) == true){

		if(info == ""){
			info = "No information to display";
		}

		if(m_ptrInfoDlg->GetSafeHwnd() == 0){
			m_ptrInfoDlg->Create();
		}
		
		m_ptrInfoDlg->info_text(info);

		postRedisplay(0, 0);
	}

	CView::OnLButtonDblClk(nFlags, point);
}


void CGVView::OnMouseMove(UINT nFlags, CPoint point) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if(pDoc->gp.mouse_move(point.x, point.y) == true){
		postRedisplay(0, 0);
	}

	// Keep the user informed of the current pointer location (in base)
	UpdateStatusBar(pDoc);

	CView::OnMouseMove(nFlags, point);
}

BOOL CGVView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if(pDoc->gp.mouse_wheel(pt.x, pt.y, zDelta) == true){

		// Keep the user informed of the current pointer location (in base)
		UpdateStatusBar(pDoc);

		postRedisplay(0, 0);
	}
	
	return CView::OnMouseWheel(nFlags, zDelta, pt);
}

void CGVView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	// Grap the mouse
	SetCapture();

	if(pDoc->gp.r_button_down(point.x, point.y) == true){
		postRedisplay(0, 0);
	}
	
	CView::OnRButtonDown(nFlags, point);
}

void CGVView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	// Give up control of the mouse
	ReleaseCapture();

	if(pDoc->gp.r_button_up(point.x, point.y) == true){
		postRedisplay(0, 0);
	}
	
	CView::OnRButtonUp(nFlags, point);
}

void CGVView::OnMButtonDown(UINT nFlags, CPoint point) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	// Grap the mouse
	SetCapture();

	if(pDoc->gp.m_button_down(point.x, point.y) == true){
		postRedisplay(0, 0);
	}
	
	CView::OnMButtonDown(nFlags, point);
}

void CGVView::OnMButtonUp(UINT nFlags, CPoint point) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	// Give up control of the mouse
	ReleaseCapture();

	if(pDoc->gp.m_button_up(point.x, point.y) == true){
		postRedisplay(0, 0);
	}
	
	CView::OnMButtonUp(nFlags, point);
}

void CGVView::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	bool update_view = false;

	switch(nChar){
		case '+':
			update_view = pDoc->gp.key_plus();
			break;
		case '-':
			update_view = pDoc->gp.key_minus();
			break;
	};
	
	if(update_view){
		// Keep the user informed of the current pointer location (in base)
		UpdateStatusBar(pDoc);

		postRedisplay(0, 0);
	}

	CView::OnChar(nChar, nRepCnt, nFlags);
}

void CGVView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	bool update_view = false;

	switch(nChar){
		case 27: // ESCAPE
			update_view = pDoc->gp.clear_selection();
			pDoc->gp.jump_buffer_clear();
			break;
		case 33: // Page up
			update_view = pDoc->gp.key_page_up();
			break;
		case 34: // Page down
			update_view = pDoc->gp.key_page_down();
			break;
		case 37: // Left arrow
			update_view = pDoc->gp.key_left_arrow();
			break;
		case 38: // Up arrow
			update_view = pDoc->gp.key_up_arrow();
			break;
		case 39: // Right arrow
			update_view = pDoc->gp.key_right_arrow();
			break;
		case 40: // Down arrow
			update_view = pDoc->gp.key_down_arrow();
			break;
		case 'G': // Shift + G
			pDoc->gp.jump_to();
			pDoc->gp.jump_buffer_clear();
			postRedisplay(0, 0);
			break;
	};

	// Allow the user to use the vi command: # + Shift G to
	// jump to a base
	if(isdigit(nChar)){
		pDoc->gp.jump_buffer_push(nChar - '0');
	}

	if(update_view){

		UpdateStatusBar(pDoc);

		postRedisplay(0, 0);
	}

	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CGVView::UpdateStatusBar(CGVDoc* m_pDoc)
{
	std::stringstream ssout;
	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;

	if(pFrame == NULL){
		return;
	}

	CStatusBar* pStatus = &pFrame->m_wndStatusBar;

	if(pStatus == NULL){
		return;
	}

	ssout << "base(" << m_pDoc->gp.cur_base_loc() << ")";

	ssout.precision(3);

	// Is the user displaying Tm in the current genome?
	if(m_pDoc->gp.cur_show_tm()){
		ssout << ":Tm(" << m_pDoc->gp.cur_tm() << ")";
	}
	
	// Is the user displaying %GC in the current genome?
	if(m_pDoc->gp.cur_show_gc()){
		ssout << ":%GC(" << m_pDoc->gp.cur_gc() << ")";
	}
	
	// Is the user displaying %AT in the current genome?
	if(m_pDoc->gp.cur_show_at()){
		ssout << ":%AT(" << m_pDoc->gp.cur_at() << ")";
	}

	// Is the user displaying GC skew in the current genome?
	if(m_pDoc->gp.cur_show_gc_skew()){
		ssout << ":GC skew(" << m_pDoc->gp.cur_gc_skew() << ")";
	}

	// Is the user displaying AT skew in the current genome?
	if(m_pDoc->gp.cur_show_at_skew()){
		ssout << ":AT skew(" << m_pDoc->gp.cur_at_skew() << ")";
	}

	// Is the user displaying keto skew in the current genome?
	if(m_pDoc->gp.cur_show_keto_skew()){
		ssout << ":keto(" << m_pDoc->gp.cur_keto_skew() << ")";
	}

	// Is the user displaying purine skew in the current genome?
	if(m_pDoc->gp.cur_show_purine_skew()){
		ssout << ":keto(" << m_pDoc->gp.cur_purine_skew() << ")";
	}

	// Is the user displaying a genome plot in the current genome?
	if(m_pDoc->gp.cur_show_genome_plot()){
		ssout << ":user(" << m_pDoc->gp.cur_genome_plot() << ")";
	}

	pStatus->SetPaneText(PLOT_PANE, ssout.str().c_str());
}

void CGVView::OnEditClear() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if(pDoc->gp.delete_selection() == true){
		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateEditClear(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnEditMoveDown() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if(pDoc->gp.shift_selection_down() == true){
		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateEditMoveDown(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnEditMoveUp() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if(pDoc->gp.shift_selection_up() == true){
		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateEditMoveUp(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnEditComplement() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	if(pDoc->gp.complement_selection() == true){
		postRedisplay(0, 0);
	}

	::SetCursor(prevCursor);
}

void CGVView::OnUpdateEditComplement(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
	
}

void CGVView::OnEditSelectAll() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.select_all();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateEditSelectAll(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_display() != 0);
}

void CGVView::OnViewShowLess() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.decrease_genome_display();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateViewShowLess(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_display() > 1);
}

void CGVView::OnViewShowMore() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.increase_genome_display();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateViewShowMore(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_display() != 0);
}

void CGVView::OnEditGc() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.toggle_gc_selection();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateEditGc(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnEditGcSkew() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.toggle_gc_skew_selection();

	postRedisplay(0, 0);
	
}

void CGVView::OnUpdateEditGcSkew(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnEditKetoSkew() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.toggle_keto_skew_selection();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateEditKetoSkew(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnEditPurineSkew() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.toggle_purine_skew_selection();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateEditPurineSkew(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnEditAt() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.toggle_at_selection();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateEditAt(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnEditAtSkew() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.toggle_at_skew_selection();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateEditAtSkew(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnEditTm() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.toggle_tm_selection();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateEditTm(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
	
}

void CGVView::OnEditGenomePlot() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.toggle_genome_plot_selection();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateEditGenomePlot(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnGenomePlotConservation() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.toggle_conservation_plot();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateGenomePlotConservation(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->SetCheck( pDoc->gp.show_conservation_plot() );
}

void CGVView::OnViewHideSelection() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if(pDoc->gp.hide_selection() == true){
		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateViewHideSelection(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnViewHideAnnotation() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if(pDoc->gp.hide_selection_annotation() == true){
		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateViewHideAnnotation(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnViewOptions() 
{
	if(m_ptrOptionsDlg->GetSafeHwnd() == 0){
		m_ptrOptionsDlg->Create();
	}
}

void CGVView::OnUpdateViewOptions(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_display() != 0);
}

void CGVView::OnViewShowAll() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if(pDoc->gp.show_all() == true){
		postRedisplay(0, 0);
	}
}

void CGVView::OnViewShowAnnotation() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if(pDoc->gp.show_annotation() == true){
		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateViewShowAnnotation(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}


void CGVView::OnFileMruFile1() 
{
	OnFileMruFile("File1");
}

void CGVView::OnFileMruFile2() 
{
	OnFileMruFile("File2");	
}

void CGVView::OnFileMruFile3() 
{
	OnFileMruFile("File3");
}

void CGVView::OnFileMruFile4()
{
	OnFileMruFile("File4");
}

void CGVView::OnFileMruFile(const char *m_mru_string)
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	HCURSOR prevCursor = NULL;

	// Open the File
	try{
		prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));
		
		CString filename = AfxGetApp()->GetProfileString("Recent File List", m_mru_string);

		if(pDoc->gp.load(LPCTSTR(filename), file_type(LPCTSTR(filename))) == false){

			AfxMessageBox("Did not read any sequence data for this genome. "
						"Try downloading the file directly from NCBI");
		}

		::SetCursor(prevCursor);
	}
	catch(const char *error){
		if(prevCursor){
			::SetCursor(prevCursor);
		}

		// Was was the error opening the file?
		AfxMessageBox(error);
		return;
	}
	catch(...){
		if(prevCursor){
			::SetCursor(prevCursor);
		}

		// Yiikes! An unhandled error has occured!
		// Was there an error opening the file?
		AfxMessageBox("Unable to read input file!");

		return;
	}

	postRedisplay(0, 0);
	
}

void CGVView::OnEditSearch() 
{
	if(m_ptrSearchDlg->GetSafeHwnd() == 0){
		m_ptrSearchDlg->Create();
	}
}

LRESULT CGVView::OnEditSearchClose(WPARAM wParam, LPARAM lParam) 
{
	if(m_ptrSearchDlg){
		m_ptrSearchDlg->DestroyWindow();
	}

	return 0L;
}

LRESULT CGVView::HybridPrimer(WPARAM wParam, LPARAM lParam)
{
	std::string search_results;

	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Make sure that there is at least one genome loaded!
	if(pDoc->gp.num_display() == 0){
		AfxMessageBox("No data loaded!");
		return 0L;
	}

	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	try{
		// Search all genomes with the given probe
		search_results = pDoc->gp.search_primers( LPCTSTR(m_ptrHybridDlg->primer1_string()),
			LPCTSTR(m_ptrHybridDlg->primer2_string()),
			m_ptrHybridDlg->salt(), m_ptrHybridDlg->strand(),
			m_ptrHybridDlg->tm(), m_ptrHybridDlg->anchor(),
			m_ptrHybridDlg->min_amplicon_length(),
			m_ptrHybridDlg->max_amplicon_length() );
	}
	catch(const char *error){
		::SetCursor(prevCursor);
		AfxMessageBox(error);
		return 0L;
	}
	catch(...){
		::SetCursor(prevCursor);
		AfxMessageBox("An unhandled search error has occured. Doh!");
		return 0L;
	}

	::SetCursor(prevCursor);

	// Display the search results
	m_ptrHybridDlg->results_string(search_results);

	// Update the display to show selected search hits (if any)
	postRedisplay(0, 0);

	return 0L;
}

LRESULT CGVView::HybridProbe(WPARAM wParam, LPARAM lParam)
{
	std::string search_results;

	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Make sure that there is at least one genome loaded!
	if(pDoc->gp.num_display() == 0){
		AfxMessageBox("No data loaded!");
		return 0L;
	}

	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	try{
		
		// Search all genomes with the given probe
		search_results = pDoc->gp.search_probe(LPCTSTR(m_ptrHybridDlg->probe_string()),
			m_ptrHybridDlg->salt(), m_ptrHybridDlg->strand(), m_ptrHybridDlg->tm());

	}
	catch(const char *error){
		::SetCursor(prevCursor);
		AfxMessageBox(error);
		return 0L;
	}
	catch(...){
		::SetCursor(prevCursor);
		AfxMessageBox("An unhandled search error has occured. Doh!");
		return 0L;
	}

	::SetCursor(prevCursor);

	// Display the search results
	m_ptrHybridDlg->results_string(search_results);

	// Update the display to show selected search hits (if any)
	postRedisplay(0, 0);

	return 0L;
}

LRESULT CGVView::MOLPCRHybridPrimer(WPARAM wParam, LPARAM lParam)
{
	std::string search_results;

	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Make sure that there is at least one genome loaded!
	if(pDoc->gp.num_display() == 0){
		AfxMessageBox("No data loaded!");
		return 0L;
	}

	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	try{
		// Search all genomes with the given probe
		search_results = pDoc->gp.search_MOL_primers( LPCTSTR(m_ptrMolPCRDlg->primer1_string()),
			LPCTSTR(m_ptrMolPCRDlg->primer2_string()),
			m_ptrMolPCRDlg->salt(), m_ptrMolPCRDlg->strand(),
			m_ptrMolPCRDlg->tm(), m_ptrMolPCRDlg->anchor() );
	}
	catch(const char *error){
		::SetCursor(prevCursor);
		AfxMessageBox(error);
		return 0L;
	}
	catch(...){
		::SetCursor(prevCursor);
		AfxMessageBox("An unhandled search error has occured. Doh!");
		return 0L;
	}

	::SetCursor(prevCursor);

	// Display the search results
	m_ptrMolPCRDlg->results_string(search_results);

	// Update the display to show selected search hits (if any)
	postRedisplay(0, 0);

	return 0L;
}

void CGVView::OnEditHybrid() 
{
	if(m_ptrHybridDlg->GetSafeHwnd() == 0){
		m_ptrHybridDlg->Create();
	}
}

LRESULT CGVView::OnEditHybridClose(WPARAM wParam, LPARAM lParam) 
{
	if(m_ptrHybridDlg){
		m_ptrHybridDlg->DestroyWindow();
	}

	return 0L;
}

void CGVView::OnEditHybridMolPcr() 
{
	if(m_ptrMolPCRDlg->GetSafeHwnd() == 0){
		m_ptrMolPCRDlg->Create();
	}
}

LRESULT CGVView::OnEditHybridMolPcrClose(WPARAM wParam, LPARAM lParam) 
{
	if(m_ptrMolPCRDlg){
		m_ptrMolPCRDlg->DestroyWindow();
	}

	return 0L;
}

LRESULT CGVView::OnInfoClose(WPARAM wParam, LPARAM lParam)
{
	if(m_ptrInfoDlg){
		m_ptrInfoDlg->DestroyWindow();
	}

	return 0L;
}

LRESULT CGVView::OnOptionsClose(WPARAM wParam, LPARAM lParam)
{
	if(m_ptrOptionsDlg){
		m_ptrOptionsDlg->DestroyWindow();
	}

	return 0L;
}

LRESULT CGVView::OnOptionsUpdate(WPARAM wParam, LPARAM lParam)
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// What event occured/value needs to be updated?
	switch(wParam){
		case WM_OPTIONS_SLIDE_FONT:
			pDoc->gp.font_scale_index(lParam);

			postRedisplay(0, 0);

			// Update the dialog as well
			if(m_ptrOptionsDlg){
				m_ptrOptionsDlg->PostMessage(WM_OPTIONS_UPDATE, WM_OPTIONS_SLIDE_FONT, 0);
			}

			break;
		case WM_OPTIONS_SLIDE_GC:
			pDoc->gp.content_window(lParam);

			postRedisplay(0, 0);

			// Update the dialog as well
			if(m_ptrOptionsDlg){
				m_ptrOptionsDlg->PostMessage(WM_OPTIONS_UPDATE, WM_OPTIONS_SLIDE_GC, 0);
			}

			break;
		case WM_OPTIONS_SLIDE_TM:
			pDoc->gp.tm_window(lParam);

			postRedisplay(0, 0);

			// Update the dialog as well
			if(m_ptrOptionsDlg){
				m_ptrOptionsDlg->PostMessage(WM_OPTIONS_UPDATE, WM_OPTIONS_SLIDE_TM, 0);
			}
			break;
		case WM_OPTIONS_SLIDE_NA:
			pDoc->gp.tm_salt_index(lParam);

			postRedisplay(0, 0);

			// Update the dialog as well
			if(m_ptrOptionsDlg){
				m_ptrOptionsDlg->PostMessage(WM_OPTIONS_UPDATE, WM_OPTIONS_SLIDE_NA, 0);
			}
			break;
		case WM_OPTIONS_SLIDE_STRAND:
			pDoc->gp.tm_strand_index(lParam);

			postRedisplay(0, 0);

			// Update the dialog as well
			if(m_ptrOptionsDlg){
				m_ptrOptionsDlg->PostMessage(WM_OPTIONS_UPDATE, WM_OPTIONS_SLIDE_STRAND, 0);
			}
			break;
	};

	return 0L;
}

LRESULT CGVView::Search(WPARAM wParam, LPARAM lParam)
{
	std::string search_results;

	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Make sure that there is at least one genome loaded!
	if(pDoc->gp.num_display() == 0){
		AfxMessageBox("No data loaded!");
		return 0L;
	}

	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	try{
		switch( m_ptrSearchDlg->key() ){

			case CSearchDialog::KEY_NAME:
				if(pDoc->gp.validate_search_key(GenomePage::SEARCH_KEY_NAME, 
					LPCTSTR(m_ptrSearchDlg->key_string())) == false){
					
					::SetCursor(prevCursor);
					AfxMessageBox("Invalid gene LOCUS/LOCUS TAG");
					return 0L;
				}

				// Search all genomes
				search_results = pDoc->gp.search(GenomePage::SEARCH_KEY_NAME, 
					LPCTSTR(m_ptrSearchDlg->key_string()));

				break;
			case CSearchDialog::KEY_GI:
				if(pDoc->gp.validate_search_key(GenomePage::SEARCH_KEY_GI, 
					LPCTSTR(m_ptrSearchDlg->key_string())) == false){

					::SetCursor(prevCursor);
					AfxMessageBox("Invalid GI");
					return 0L;
				}

				search_results = pDoc->gp.search(GenomePage::SEARCH_KEY_GI, 
					LPCTSTR(m_ptrSearchDlg->key_string()));

				break;
			case CSearchDialog::KEY_BASE:
				if(pDoc->gp.validate_search_key(GenomePage::SEARCH_KEY_BASE, 
					LPCTSTR(m_ptrSearchDlg->key_string())) == false){

					::SetCursor(prevCursor);
					AfxMessageBox("Invalid base position");
					return 0L;
				}
				search_results = pDoc->gp.search(GenomePage::SEARCH_KEY_BASE, 
					LPCTSTR(m_ptrSearchDlg->key_string()));

				break;
			case CSearchDialog::KEY_WORD:
				if(pDoc->gp.validate_search_key(GenomePage::SEARCH_KEY_TEXT, 
					LPCTSTR(m_ptrSearchDlg->key_string())) == false){

					::SetCursor(prevCursor);
					AfxMessageBox("Invalid text string");
					return 0L;
				}

				search_results = pDoc->gp.search(GenomePage::SEARCH_KEY_TEXT, 
					LPCTSTR(m_ptrSearchDlg->key_string()));

				break;
			case CSearchDialog::KEY_DNA:
				if(pDoc->gp.validate_search_key(GenomePage::SEARCH_KEY_DNA, 
					LPCTSTR(m_ptrSearchDlg->key_string())) == false){

					::SetCursor(prevCursor);
					AfxMessageBox("Invalid DNA sequence");
					return 0L;
				}

				search_results = pDoc->gp.search(GenomePage::SEARCH_KEY_DNA, 
					LPCTSTR(m_ptrSearchDlg->key_string()));

				break;
			case CSearchDialog::KEY_GENOME_NAME:
				if(pDoc->gp.validate_search_key(GenomePage::SEARCH_KEY_GENOME_NAME, 
					LPCTSTR(m_ptrSearchDlg->key_string())) == false){

					::SetCursor(prevCursor);
					AfxMessageBox("Invalid DNA sequence");
					return 0L;
				}

				search_results = pDoc->gp.search(GenomePage::SEARCH_KEY_GENOME_NAME, 
					LPCTSTR(m_ptrSearchDlg->key_string()));

				break;
		};
	}
	catch(const char *error){
		::SetCursor(prevCursor);
		AfxMessageBox(error);
		return 0L;
	}
	catch(...){
		::SetCursor(prevCursor);
		AfxMessageBox("An unhandled search error has occured. Doh!");
		return 0L;
	}

	::SetCursor(prevCursor);

	// Display the search results
	m_ptrSearchDlg->results_string(search_results);

	// Update the display to show selected search hits (if any)
	postRedisplay(0, 0);

	return 0L;
}

LRESULT CGVView::ShowNextSearch(WPARAM wParam, LPARAM lParam)
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	pDoc->gp.search_next();

	::SetCursor(prevCursor);

	postRedisplay(0, 0);

	return 0L;
}

void CGVView::OnEditSearchAgain() 
{
	ShowNextSearch(0, 0);
}

LRESULT CGVView::AnnotateSelection(WPARAM wParam, LPARAM lParam)
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	bool ret = pDoc->gp.annotate_selection();

	::SetCursor(prevCursor);

	if(ret == false){
		AfxMessageBox("There must be at least one selected sequence region to annotate");
	}

	postRedisplay(0, 0);

	return 0L;
}

LRESULT CGVView::PrimerClose(WPARAM wParam, LPARAM lParam)
{
	if(m_ptrPrimerDlg){
		m_ptrPrimerDlg->DestroyWindow();
	}

	return 0L;
}

afx_msg LRESULT CGVView::PrimerOk(WPARAM wParam, LPARAM lParam)
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Make sure that there is at least one genome loaded!
	if(pDoc->gp.num_select() == 0){
		AfxMessageBox("Please select at least one genome");
		return 0L;
	}
	
	HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

	try{
		
		// Set the options mask
		unsigned int mask = 0;

		if(m_ptrPrimerDlg->m_option_gc_content){
			mask |= GC_CONTENT;
		}

		if(m_ptrPrimerDlg->m_option_multi_gc){
			mask |= MULTI_5_GC;
		}
		
		if(m_ptrPrimerDlg->m_option_poly_gc){
			mask |= POLY_3_GC;
		}
		
		if(m_ptrPrimerDlg->m_option_poly_run){
			mask |= NO_POLY_RUNS;
		}
		
		if(m_ptrPrimerDlg->m_option_t_3){
			mask |= NO_3_T;
		}

		// Pick primers for the selected genomes
		pDoc->gp.compute_primers(mask,
			m_ptrPrimerDlg->m_primer_len,
			m_ptrPrimerDlg->m_run_len,
			m_ptrPrimerDlg->m_salt,
			m_ptrPrimerDlg->m_strand,
			m_ptrPrimerDlg->m_min_gc,
			m_ptrPrimerDlg->m_max_gc,
			m_ptrPrimerDlg->m_tm,
			m_ptrPrimerDlg->m_mis_prime_tm);

	}
	catch(const char *error){
		::SetCursor(prevCursor);
		AfxMessageBox(error);
		return 0L;
	}
	catch(...){
		::SetCursor(prevCursor);
		AfxMessageBox("An unhandled primer error has occured. Doh!");
		return 0L;
	}

	::SetCursor(prevCursor);

	postRedisplay(0, 0);

	return 0L;
}

LRESULT CGVView::AnnotationClose(WPARAM wParam, LPARAM lParam)
{
	if(m_ptrAnnotDlg){

		m_ptrAnnotDlg->DestroyWindow();
	}

	return 0L;
}

afx_msg LRESULT CGVView::AnnotationApply(WPARAM wParam, LPARAM lParam)
{
	if(m_ptrAnnotDlg){
		CGVDoc* pDoc = GetDocument();
		ASSERT_VALID(pDoc);

		pDoc->gp.update_annotation(m_ptrAnnotDlg->edit_annot);

		postRedisplay(0, 0);
	}

	return 0L;
}

void CGVView::OnGenomePrimer() 
{
	if(m_ptrPrimerDlg->GetSafeHwnd() == 0){
		m_ptrPrimerDlg->Create();
	}
}

void CGVView::OnUpdateGenomePrimer(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnGenomePrimerClear() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Make sure that there is at least one genome loaded!
	if(pDoc->gp.num_select() == 0){
		AfxMessageBox("Please select at least one genome");
		return;
	}

	pDoc->gp.clear_primers();

	postRedisplay(0, 0);
}

void CGVView::OnUpdateGenomePrimerClear(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnFileExport() 
{
	HCURSOR prevCursor = NULL;

	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	CFileDialog fDlg(FALSE); // Save as ...

	fDlg.m_ofn.lpstrTitle = "Export Image";
	fDlg.m_ofn.lpstrFilter = "PostScript file\0*.ps\0GIF file\0*.gif\0All Files\0*.*\0\0";

	if(fDlg.DoModal() == IDOK){
		try{
			prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));
			
			switch(fDlg.m_ofn.nFilterIndex){
				case 1: // Postscript
					pDoc->gp.render_ps(LPCSTR(fDlg.GetFileName()));
					break;
				case 2: // GIF
					pDoc->gp.render_gif(LPCSTR(fDlg.GetFileName()));
					break;
			};

			::SetCursor(prevCursor);
		}
		catch(const char *error){
			if(prevCursor){
				::SetCursor(prevCursor);
			}

			AfxMessageBox(error);
		}
		catch(...){
			if(prevCursor){
				::SetCursor(prevCursor);
			}

			AfxMessageBox("Unable to render image. Unknown error");
		}
	}
}


void CGVView::OnColorBackground() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	CColorDialog colorDlg;

	colorDlg.m_cc.Flags |= CC_RGBINIT;

	// Set the default color choice
	colorDlg.m_cc.rgbResult = RGB(GV_DEFAULT_BACKGROUND_R,
								  GV_DEFAULT_BACKGROUND_G,
								  GV_DEFAULT_BACKGROUND_B);

	if(colorDlg.DoModal() == IDOK){

		pDoc->gp.background_color(GetRValue(colorDlg.m_cc.rgbResult),
								  GetGValue(colorDlg.m_cc.rgbResult),
								  GetBValue(colorDlg.m_cc.rgbResult));

		postRedisplay(0, 0);
	}
}

void CGVView::OnColorAnnot() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CColorDialog colorDlg;

	colorDlg.m_cc.Flags |= CC_RGBINIT;

	// Set the default color choice
	colorDlg.m_cc.rgbResult = RGB(GV_DEFAULT_ANNOT_R,
								  GV_DEFAULT_ANNOT_G,
								  GV_DEFAULT_ANNOT_B);

	if(colorDlg.DoModal() == IDOK){

		pDoc->gp.annot_color(GetRValue(colorDlg.m_cc.rgbResult),
							 GetGValue(colorDlg.m_cc.rgbResult),
							 GetBValue(colorDlg.m_cc.rgbResult));

		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateColorAnnot(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnColorPrimer() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CColorDialog colorDlg;

	colorDlg.m_cc.Flags |= CC_RGBINIT;

	// Set the default color choice
	colorDlg.m_cc.rgbResult = RGB(GV_DEFAULT_PRIMER_R,
								  GV_DEFAULT_PRIMER_G,
								  GV_DEFAULT_PRIMER_B);

	if(colorDlg.DoModal() == IDOK){

		pDoc->gp.primer_color(GetRValue(colorDlg.m_cc.rgbResult),
							  GetGValue(colorDlg.m_cc.rgbResult),
							  GetBValue(colorDlg.m_cc.rgbResult));

		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateColorPrimer(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
	
}

void CGVView::OnColorCds() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CColorDialog colorDlg;

	colorDlg.m_cc.Flags |= CC_RGBINIT;

	// Set the default color choice
	colorDlg.m_cc.rgbResult = RGB(GV_DEFAULT_CDS_R,
								  GV_DEFAULT_CDS_G,
								  GV_DEFAULT_CDS_B);

	if(colorDlg.DoModal() == IDOK){

		pDoc->gp.cds_color(GetRValue(colorDlg.m_cc.rgbResult),
						   GetGValue(colorDlg.m_cc.rgbResult),
						   GetBValue(colorDlg.m_cc.rgbResult));

		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateColorCds(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnColorGene() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CColorDialog colorDlg;

	colorDlg.m_cc.Flags |= CC_RGBINIT;

	// Set the default color choice
	colorDlg.m_cc.rgbResult = RGB(GV_DEFAULT_GENE_R,
								  GV_DEFAULT_GENE_G,
								  GV_DEFAULT_GENE_B);

	if(colorDlg.DoModal() == IDOK){

		pDoc->gp.gene_color(GetRValue(colorDlg.m_cc.rgbResult),
						    GetGValue(colorDlg.m_cc.rgbResult),
						    GetBValue(colorDlg.m_cc.rgbResult));

		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateColorGene(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnColorPseudoGene() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CColorDialog colorDlg;

	colorDlg.m_cc.Flags |= CC_RGBINIT;

	// Set the default color choice
	colorDlg.m_cc.rgbResult = RGB(GV_DEFAULT_PSEUDO_GENE_R,
								  GV_DEFAULT_PSEUDO_GENE_G,
								  GV_DEFAULT_PSEUDO_GENE_B);

	if(colorDlg.DoModal() == IDOK){

		pDoc->gp.pseudo_gene_color(GetRValue(colorDlg.m_cc.rgbResult),
						    GetGValue(colorDlg.m_cc.rgbResult),
						    GetBValue(colorDlg.m_cc.rgbResult));

		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateColorPseudoGene(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnColorIntergenic() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CColorDialog colorDlg;

	colorDlg.m_cc.Flags |= CC_RGBINIT;

	// Set the default color choice
	colorDlg.m_cc.rgbResult = RGB(GV_DEFAULT_INTERGENIC_R,
								  GV_DEFAULT_INTERGENIC_G,
								  GV_DEFAULT_INTERGENIC_B);

	if(colorDlg.DoModal() == IDOK){

		pDoc->gp.intergenic_color(GetRValue(colorDlg.m_cc.rgbResult),
								  GetGValue(colorDlg.m_cc.rgbResult),
								  GetBValue(colorDlg.m_cc.rgbResult));

		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateColorIntergenic(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnColorRna() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CColorDialog colorDlg;

	colorDlg.m_cc.Flags |= CC_RGBINIT;

	// Set the default color choice
	colorDlg.m_cc.rgbResult = RGB(GV_DEFAULT_RNA_R,
								  GV_DEFAULT_RNA_G,
								  GV_DEFAULT_RNA_B);

	if(colorDlg.DoModal() == IDOK){

		pDoc->gp.rna_color(GetRValue(colorDlg.m_cc.rgbResult),
						   GetGValue(colorDlg.m_cc.rgbResult),
						   GetBValue(colorDlg.m_cc.rgbResult));

		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateColorRna(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnColorSelect() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CColorDialog colorDlg;

	colorDlg.m_cc.Flags |= CC_RGBINIT;

	// Set the default color choice
	colorDlg.m_cc.rgbResult = RGB(GV_DEFAULT_SELECT_R,
								  GV_DEFAULT_SELECT_G,
								  GV_DEFAULT_SELECT_B);

	if(colorDlg.DoModal() == IDOK){

		pDoc->gp.select_color(GetRValue(colorDlg.m_cc.rgbResult),
						      GetGValue(colorDlg.m_cc.rgbResult),
						      GetBValue(colorDlg.m_cc.rgbResult));

		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateColorSelect(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnColorText() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CColorDialog colorDlg;

	colorDlg.m_cc.Flags |= CC_RGBINIT;

	// Set the default color choice
	colorDlg.m_cc.rgbResult = RGB(GV_DEFAULT_TEXT_R,
								  GV_DEFAULT_TEXT_G,
								  GV_DEFAULT_TEXT_B);

	if(colorDlg.DoModal() == IDOK){

		pDoc->gp.text_color(GetRValue(colorDlg.m_cc.rgbResult),
						    GetGValue(colorDlg.m_cc.rgbResult),
						    GetBValue(colorDlg.m_cc.rgbResult));

		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateColorText(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnColorCustom() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	CColorDialog colorDlg;

	colorDlg.m_cc.Flags |= CC_RGBINIT;

	// Set the default color choice
	colorDlg.m_cc.rgbResult = RGB(GV_DEFAULT_TEXT_R,
								  GV_DEFAULT_TEXT_G,
								  GV_DEFAULT_TEXT_B);

	if(colorDlg.DoModal() == IDOK){

		pDoc->gp.custom_color(GetRValue(colorDlg.m_cc.rgbResult),
						      GetGValue(colorDlg.m_cc.rgbResult),
						      GetBValue(colorDlg.m_cc.rgbResult));

		postRedisplay(0, 0);
	}
}

void CGVView::OnUpdateColorCustom(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnSequenceShowAa() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	std::string info;

	info = pDoc->gp.selected_seq_as_protein();

	if(info != ""){
		if(m_ptrInfoDlg->GetSafeHwnd() == 0){
			m_ptrInfoDlg->Create();
		}
					
		m_ptrInfoDlg->info_text(info);	
	}
}

void CGVView::OnUpdateSequenceShowAa(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnSequenceShowNa() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	std::string info;

	info = pDoc->gp.selected_seq_as_dna();

	if(info != ""){
		if(m_ptrInfoDlg->GetSafeHwnd() == 0){
			m_ptrInfoDlg->Create();
		}
					
		m_ptrInfoDlg->info_text(info);	
	}
}

void CGVView::OnUpdateSequenceShowNa(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnFileGenomePlot() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Open a file dialog box and let the user specify a text
	// file

	CFileDialog fDlg(TRUE);

	fDlg.m_ofn.lpstrFilter = "All Files\0*.*\0\0";

	// Allow the user to open multiple files
	// Allow the user to select multiple files [so we must construct a 
	// string buffer to hold the returned name(s)].
	fDlg.m_ofn.nMaxFile = 2562; // <-- Windows NT 4.0 max. Dooh!

	fDlg.m_ofn.Flags = fDlg.m_ofn.Flags | OFN_ALLOWMULTISELECT;

	// Save a pointer to the buffer for easy memory clean up!
	char *buf;

	fDlg.m_ofn.lpstrFile = buf = new char [fDlg.m_ofn.nMaxFile];

	if(buf == NULL){
		AfxMessageBox("Memory allocation error!");
		return;
	}

	// Set the first character to null to prevent a default filename from 
	// appearing.
	fDlg.m_ofn.lpstrFile[0] = '\0';

	// Set the current directory
	char dirBuffer[1024];

	// If the buffer is not big enought, DON'T set the current 
	// directory
	if(GetCurrentDirectory(1023, dirBuffer) != 0){
		fDlg.m_ofn.lpstrInitialDir = dirBuffer;
	}

	if(fDlg.DoModal() == IDOK){
		if(CommDlgExtendedError() == FNERR_BUFFERTOOSMALL){
			AfxMessageBox("Too many files selected! "
				"Please select fewer files and try again.");

			delete [] buf;

			return;
		}

		POSITION pos = fDlg.GetStartPosition();
		CString file;
		int numFilesRead = 0;

		HCURSOR prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));

		while(pos){
			numFilesRead++;
			file = fDlg.GetNextPathName(pos);

			// Open the File
			try{
				pDoc->gp.read_genome_plot( LPCTSTR(file) );
			}
			catch(const char *error){
				::SetCursor(prevCursor);

				// Was there an error opening the file?
				AfxMessageBox(error);
				
				delete [] buf;

				return;
			}
			catch(...){
				::SetCursor(prevCursor);

				// Yiikes! An unhandled error has occured!
				// Was there an error opening the file?
				AfxMessageBox("Unable to read input file!");
				
				delete [] buf;

				return;
			}
		}

		::SetCursor(prevCursor);

		if(numFilesRead == 0){
			delete [] buf;

			AfxMessageBox("Please enter a file name");
			return ;
		}

		delete [] buf;

		postRedisplay(0, 0);
	}
}


void CGVView::OnViewSizeCentric() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.mode(DNAMol::SIZE_CENTRIC);

	postRedisplay(0, 0);
}

void CGVView::OnUpdateViewSizeCentric(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->SetCheck(pDoc->gp.mode() == DNAMol::SIZE_CENTRIC);
}

void CGVView::OnViewGeneCentric() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pDoc->gp.mode(DNAMol::GENE_CENTRIC);

	postRedisplay(0, 0);
}

void CGVView::OnUpdateViewGeneCentric(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->SetCheck(pDoc->gp.mode() == DNAMol::GENE_CENTRIC);
}


void CGVView::OnGenomeDelAnnot() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	if(pDoc->gp.delete_annotation() == true){
		postRedisplay(0, 0);	
	}
	else{
		AfxMessageBox("Please select at least one annotation to delete");
	}
}

void CGVView::OnUpdateGenomeDelAnnot(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnViewMomentum() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Toggle the momentum flag
	pDoc->gp.spin_momentum( !pDoc->gp.spin_momentum() );
}

void CGVView::OnUpdateViewMomentum(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pCmdUI->SetCheck(pDoc->gp.spin_momentum() == true);
}


void CGVView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	const unsigned int delta_line = 1;
	const unsigned int delta_page = 10;

	switch(nSBCode){
		case SB_LINELEFT:
			pDoc->gp.scroll_bar(HSCROLL_DEFAULT_POS - delta_line, HSCROLL_MAX - HSCROLL_MIN);
			pDoc->gp.scroll_bar(HSCROLL_DEFAULT_POS);
			CView::SetScrollPos(SB_HORZ, HSCROLL_DEFAULT_POS, false);
			break;
		case SB_LINERIGHT:
			pDoc->gp.scroll_bar(HSCROLL_DEFAULT_POS + delta_line, HSCROLL_MAX - HSCROLL_MIN);
			pDoc->gp.scroll_bar(HSCROLL_DEFAULT_POS);
			CView::SetScrollPos(SB_HORZ, HSCROLL_DEFAULT_POS, false);
			break;
		case SB_PAGELEFT:
			pDoc->gp.scroll_bar(HSCROLL_DEFAULT_POS - delta_page, HSCROLL_MAX - HSCROLL_MIN);
			pDoc->gp.scroll_bar(HSCROLL_DEFAULT_POS);
			CView::SetScrollPos(SB_HORZ, HSCROLL_DEFAULT_POS, false);
			break;
		case SB_PAGERIGHT:
			pDoc->gp.scroll_bar(HSCROLL_DEFAULT_POS + delta_page, HSCROLL_MAX - HSCROLL_MIN);
			pDoc->gp.scroll_bar(HSCROLL_DEFAULT_POS);
			CView::SetScrollPos(SB_HORZ, HSCROLL_DEFAULT_POS, false);
			break;
		case SB_THUMBPOSITION:
		case SB_THUMBTRACK:
			pDoc->gp.scroll_bar(nPos, HSCROLL_MAX - HSCROLL_MIN);
			break;
		case SB_ENDSCROLL:
			pDoc->gp.scroll_bar(HSCROLL_DEFAULT_POS);
			CView::SetScrollPos(SB_HORZ, HSCROLL_DEFAULT_POS, false);
			break;
	};
	
	CView::OnHScroll(nSBCode, nPos, pScrollBar);

	// Always update after a scoll
	postRedisplay(0, 0);
}

void CGVView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	const unsigned int delta_line = 1;
	const unsigned int delta_page = 10;

	switch(nSBCode){
		case SB_LINELEFT:
			pDoc->gp.zoom_bar(VSCROLL_DEFAULT_POS - delta_line, VSCROLL_MAX - VSCROLL_MIN);
			pDoc->gp.zoom_bar(VSCROLL_DEFAULT_POS);
			CView::SetScrollPos(SB_VERT, VSCROLL_DEFAULT_POS, false);
			break;
		case SB_LINERIGHT:
			pDoc->gp.zoom_bar(VSCROLL_DEFAULT_POS + delta_line, VSCROLL_MAX - VSCROLL_MIN);
			pDoc->gp.zoom_bar(VSCROLL_DEFAULT_POS);
			CView::SetScrollPos(SB_VERT, VSCROLL_DEFAULT_POS, false);
			break;
		case SB_PAGELEFT:
			pDoc->gp.zoom_bar(VSCROLL_DEFAULT_POS - delta_page, VSCROLL_MAX - VSCROLL_MIN);
			pDoc->gp.zoom_bar(VSCROLL_DEFAULT_POS);
			CView::SetScrollPos(SB_VERT, VSCROLL_DEFAULT_POS, false);
			break;
		case SB_PAGERIGHT:
			pDoc->gp.zoom_bar(VSCROLL_DEFAULT_POS + delta_page, VSCROLL_MAX - VSCROLL_MIN);
			pDoc->gp.zoom_bar(VSCROLL_DEFAULT_POS);
			CView::SetScrollPos(SB_VERT, VSCROLL_DEFAULT_POS, false);
			break;
		case SB_THUMBPOSITION:
		case SB_THUMBTRACK:
			pDoc->gp.zoom_bar(nPos, VSCROLL_MAX - VSCROLL_MIN);
			break;
		case SB_ENDSCROLL:
			pDoc->gp.zoom_bar(VSCROLL_DEFAULT_POS);
			CView::SetScrollPos(SB_VERT, VSCROLL_DEFAULT_POS, false);
			break;
	};
	
	CView::OnVScroll(nSBCode, nPos, pScrollBar);

	// Always update after a scoll
	postRedisplay(0, 0);
}

void CGVView::OnFileExportGenome() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	// The export dialog does all of the work
	CDialogExport expDlg(pDoc->gp);

	expDlg.DoModal();
}

void CGVView::OnUpdateFileExport(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_display() != 0);
}

void CGVView::OnUpdateFileExportGenome(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_display() != 0);
}


void CGVView::OnGenomeEditAnnot() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	if(m_ptrAnnotDlg->GetSafeHwnd() == 0){

		m_ptrAnnotDlg->Create();
	}
	
	m_ptrAnnotDlg->refresh_dialog(pDoc->gp);
}

void CGVView::OnUpdateGenomeEditAnnot(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnGenomeImportAnnot() 
{
	HCURSOR prevCursor = NULL;

	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	CFileDialog fDlg(TRUE); // Open.

	fDlg.m_ofn.lpstrTitle = "Import annotations";
	fDlg.m_ofn.lpstrFilter = "ASN.1 files\0*.asn;*.val;*.bna\0"
							 "GenBank files\0*.gbk;*.gbff\0"
							 "EMBL files\0*.embl\0"
							 "Protein Table files\0*.ptt\0"
							 "All Files\0*.*\0\0";
	
	if(fDlg.DoModal() == IDOK){

		try{
			prevCursor = ::SetCursor(LoadCursor(NULL, IDC_WAIT));
			
			if(pDoc->gp.import_annotation( LPCSTR(fDlg.GetFileName()) ) == true){
				postRedisplay(0, 0);
			}

			::SetCursor(prevCursor);
		}
		catch(const char *error){
			if(prevCursor){
				::SetCursor(prevCursor);
			}

			AfxMessageBox(error);
		}
		catch(...){
			if(prevCursor){
				::SetCursor(prevCursor);
			}

			AfxMessageBox("Unable to render image. Unknown error");
		}
	}
}

void CGVView::OnUpdateGenomeImportAnnot(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	
	pCmdUI->Enable(pDoc->gp.num_select() != 0);
}

void CGVView::OnViewRestore() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Toggle the momentum flag
	pDoc->gp.fit_to_window();
	postRedisplay(0, 0);
}

void CGVView::OnUpdateViewRestore(CCmdUI* pCmdUI) 
{
	// TODO: Add your command update UI handler code here
	
}

void CGVView::OnViewMagnify() 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// Toggle the momentum flag
	pDoc->gp.magnify( !pDoc->gp.magnify() );
	postRedisplay(0, 0);
}

void CGVView::OnUpdateViewMagnify(CCmdUI* pCmdUI) 
{
	CGVDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	pCmdUI->SetCheck(pDoc->gp.magnify() == true);
}
