// This program was prepared by the Regents of the University of California at 
// Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36 
// with the U.S. Department of Energy (DOE). All rights in the program are reserved 
// by the DOE and the University.  Permission is granted to the public to copy and 
// use this software without charge, provided that this Notice and any statement of 
// authorship are reproduced on all copies.  Neither the U.S. Government nor the 
// University makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this software.

// GVView.h : interface of the CGVView class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_GVVIEW_H__99781F8C_E478_46A4_B5D3_40F913937570__INCLUDED_)
#define AFX_GVVIEW_H__99781F8C_E478_46A4_B5D3_40F913937570__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "SearchDialog.h"
#include "HybridDialog.h"
#include "MolPCRHybridDialog.h"
#include "InfoDialog.h"
#include "OptionsDialogEx.h"
#include "PrimerDialog.h"
#include "AnnotationDialog.h"

#define	 HSCROLL_MIN			0
#define	 HSCROLL_MAX			100
#define	 HSCROLL_DEFAULT_POS	50

#define	 VSCROLL_MIN			0
#define	 VSCROLL_MAX			100
#define	 VSCROLL_DEFAULT_POS	50

class CGVView : public CView
{
protected: // create from serialization only
	CGVView();
	DECLARE_DYNCREATE(CGVView)

// Attributes
public:
	CGVDoc* GetDocument();

	// Manage OpenGL
	BOOL SetupPixelFormat();
	//BOOL InitializeOpenGL();
	GLuint GenerateBitmapListForFont(LOGFONT &font);
	GLuint GeneratePolygonListForFont(LOGFONT &font);
	GLuint GetNewDisplayListIDs(GLsizei range);
	void OnFileMruFile(const char *m_mru_string);

// Operations
public:
	// Screen dimensions
	unsigned int viewWidth;
	unsigned int viewHeight;

	// Bits-per-pixel for the OpenGL subsystem
	int bpp;

	// Number of bits to use for the OpenGL Z buffer
	int zbuf;

	// The ID of the OpenGL call list for displaying fonts
	GLint font_list_id;

	// A Windows font structure for storing the current font.
	LOGFONT font;
	float font_res;

	// Font scaling info
	int printScaleMul, printScaleDiv;

	HGLRC	m_hRC;
	CDC*	m_pDC;

	// Should we write a font file to disk?
	bool create_font_file;
private:
	void write_font(const std::string &m_filename, GLYPHMETRICSFLOAT agmf[], 
		const unsigned char &m_first, const unsigned char &m_last);
	void UpdateStatusBar(CGVDoc* m_pDoc);
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CGVView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	//}}AFX_VIRTUAL
private:
	// Modeless dialog boxes
	CSearchDialog *m_ptrSearchDlg;
	CHybridDialog *m_ptrHybridDlg;
	MolPCRHybridDialog *m_ptrMolPCRDlg;
	CInfoDialog *m_ptrInfoDlg;
	COptionsDialogEx *m_ptrOptionsDlg;
	CPrimerDialog *m_ptrPrimerDlg;
	CAnnotationDialog *m_ptrAnnotDlg;

// Implementation
public:
	virtual ~CGVView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CGVView)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg BOOL OnEraseBkgnd(CDC* pDC);
	afx_msg void OnFileOpen();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnMButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnEditClear();
	afx_msg void OnUpdateEditClear(CCmdUI* pCmdUI);
	afx_msg void OnEditMoveDown();
	afx_msg void OnUpdateEditMoveDown(CCmdUI* pCmdUI);
	afx_msg void OnEditMoveUp();
	afx_msg void OnUpdateEditMoveUp(CCmdUI* pCmdUI);
	afx_msg void OnEditComplement();
	afx_msg void OnUpdateEditComplement(CCmdUI* pCmdUI);
	afx_msg void OnFileMruFile1();
	afx_msg void OnFileMruFile2();
	afx_msg void OnFileMruFile3();
	afx_msg void OnFileMruFile4();
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnEditSearch();
	afx_msg void OnEditSearchAgain();
	afx_msg void OnFileExport();
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnEditSelectAll();
	afx_msg void OnUpdateEditSelectAll(CCmdUI* pCmdUI);
	afx_msg void OnEditGc();
	afx_msg void OnUpdateEditGc(CCmdUI* pCmdUI);
	afx_msg void OnViewHideSelection();
	afx_msg void OnUpdateViewHideSelection(CCmdUI* pCmdUI);
	afx_msg void OnViewShowAll();
	afx_msg void OnColorBackground();
	afx_msg void OnColorAnnot();
	afx_msg void OnUpdateColorAnnot(CCmdUI* pCmdUI);
	afx_msg void OnColorCds();
	afx_msg void OnUpdateColorCds(CCmdUI* pCmdUI);
	afx_msg void OnColorGene();
	afx_msg void OnUpdateColorGene(CCmdUI* pCmdUI);
	afx_msg void OnColorIntergenic();
	afx_msg void OnUpdateColorIntergenic(CCmdUI* pCmdUI);
	afx_msg void OnColorRna();
	afx_msg void OnUpdateColorRna(CCmdUI* pCmdUI);
	afx_msg void OnColorSelect();
	afx_msg void OnUpdateColorSelect(CCmdUI* pCmdUI);
	afx_msg void OnColorText();
	afx_msg void OnUpdateColorText(CCmdUI* pCmdUI);
	afx_msg void OnFileNcbiAsn();
	afx_msg void OnEditTm();
	afx_msg void OnUpdateEditTm(CCmdUI* pCmdUI);
	afx_msg void OnViewShowLess();
	afx_msg void OnUpdateViewShowLess(CCmdUI* pCmdUI);
	afx_msg void OnViewShowMore();
	afx_msg void OnUpdateViewShowMore(CCmdUI* pCmdUI);
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnSequenceShowAa();
	afx_msg void OnUpdateSequenceShowAa(CCmdUI* pCmdUI);
	afx_msg void OnSequenceShowNa();
	afx_msg void OnUpdateSequenceShowNa(CCmdUI* pCmdUI);
	afx_msg void OnFileGenomePlot();
	afx_msg void OnEditGenomePlot();
	afx_msg void OnUpdateEditGenomePlot(CCmdUI* pCmdUI);
	afx_msg void OnViewOptions();
	afx_msg void OnUpdateViewOptions(CCmdUI* pCmdUI);
	afx_msg void OnEditHybrid();
	afx_msg void OnEditGcSkew();
	afx_msg void OnEditKetoSkew();
	afx_msg void OnEditPurineSkew();
	afx_msg void OnEditAt();
	afx_msg void OnEditAtSkew();
	afx_msg void OnUpdateEditAt(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditAtSkew(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditGcSkew(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditKetoSkew(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditPurineSkew(CCmdUI* pCmdUI);
	afx_msg void OnColorCustom();
	afx_msg void OnUpdateColorCustom(CCmdUI* pCmdUI);
	afx_msg void OnColorPseudoGene();
	afx_msg void OnUpdateColorPseudoGene(CCmdUI* pCmdUI);
	afx_msg void OnGenomePrimer();
	afx_msg void OnUpdateGenomePrimer(CCmdUI* pCmdUI);
	afx_msg void OnGenomePrimerClear();
	afx_msg void OnUpdateGenomePrimerClear(CCmdUI* pCmdUI);
	afx_msg void OnColorPrimer();
	afx_msg void OnUpdateColorPrimer(CCmdUI* pCmdUI);
	afx_msg void OnViewSizeCentric();
	afx_msg void OnUpdateViewSizeCentric(CCmdUI* pCmdUI);
	afx_msg void OnViewGeneCentric();
	afx_msg void OnUpdateViewGeneCentric(CCmdUI* pCmdUI);
	afx_msg void OnViewHideAnnotation();
	afx_msg void OnUpdateViewHideAnnotation(CCmdUI* pCmdUI);
	afx_msg void OnViewShowAnnotation();
	afx_msg void OnUpdateViewShowAnnotation(CCmdUI* pCmdUI);
	afx_msg void OnGenomeDelAnnot();
	afx_msg void OnUpdateGenomeDelAnnot(CCmdUI* pCmdUI);
	afx_msg void OnViewMomentum();
	afx_msg void OnUpdateViewMomentum(CCmdUI* pCmdUI);
	afx_msg void OnGenomePlotConservation();
	afx_msg void OnUpdateGenomePlotConservation(CCmdUI* pCmdUI);
	afx_msg void OnEditHybridMolPcr();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnFileExportGenome();
	afx_msg void OnUpdateFileExport(CCmdUI* pCmdUI);
	afx_msg void OnUpdateFileExportGenome(CCmdUI* pCmdUI);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnGenomeEditAnnot();
	afx_msg void OnUpdateGenomeEditAnnot(CCmdUI* pCmdUI);
	afx_msg void OnGenomeImportAnnot();
	afx_msg void OnUpdateGenomeImportAnnot(CCmdUI* pCmdUI);
	afx_msg void OnViewRestore();
	afx_msg void OnUpdateViewRestore(CCmdUI* pCmdUI);
	afx_msg void OnViewMagnify();
	afx_msg void OnUpdateViewMagnify(CCmdUI* pCmdUI);
	//}}AFX_MSG
	afx_msg LRESULT OnInfoClose(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnOptionsClose(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnOptionsUpdate(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT Search(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT ShowNextSearch(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT AnnotateSelection(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnEditSearchClose(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT HybridPrimer(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT HybridProbe(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnEditHybridClose(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT PrimerClose(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT PrimerOk(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT AnnotationClose(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT AnnotationApply(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnEditHybridMolPcrClose(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT MOLPCRHybridPrimer(WPARAM wParam, LPARAM lParam);
	
	afx_msg LRESULT postRedisplay(WPARAM wParam, LPARAM lParam);

	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in GVView.cpp
inline CGVDoc* CGVView::GetDocument()
   { return (CGVDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_GVVIEW_H__99781F8C_E478_46A4_B5D3_40F913937570__INCLUDED_)

