// This program was prepared by the Regents of the University of California at 
// Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36 
// with the U.S. Department of Energy (DOE). All rights in the program are reserved 
// by the DOE and the University.  Permission is granted to the public to copy and 
// use this software without charge, provided that this Notice and any statement of 
// authorship are reproduced on all copies.  Neither the U.S. Government nor the 
// University makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this software.
//
// HybridDialog.cpp : implementation file
//

#include "stdafx.h"
#include "GV.h"
#include "nuc_cruc.h"

#include "HybridDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CHybridDialog dialog


CHybridDialog::CHybridDialog(CView* pParent)
	: CDialog(CHybridDialog::IDD, pParent)
{

	m_pView = pParent;

	//{{AFX_DATA_INIT(CHybridDialog)
	m_salt = NC_Na;
	m_strand = NC_CT;
	m_tm = 50.0f;
	m_primer1 = _T("");
	m_primer2 = _T("");
	m_probe = _T("");
	m_anchor = 3;
	m_results = _T("");
	m_max_amplicon_length = 2000;
	m_min_amplicon_length = 0;
	//}}AFX_DATA_INIT
}


void CHybridDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CHybridDialog)
	DDX_Text(pDX, IDC_SALT_EDIT, m_salt);
	DDX_Text(pDX, IDC_STRAND_EDIT, m_strand);
	DDX_Text(pDX, IDC_TM_EDIT, m_tm);
	DDX_Text(pDX, IDC_PRIMER1_EDIT, m_primer1);
	DDX_Text(pDX, IDC_PRIMER2_EDIT, m_primer2);
	DDX_Text(pDX, IDC_PROBE_EDIT, m_probe);
	DDX_Text(pDX, IDC_EXACT_EDIT, m_anchor);
	DDX_Text(pDX, IDC_HYBRID_RESULTS_RICHEDIT, m_results);
	DDX_Text(pDX, IDC_AMPLICON_LEN_EDIT, m_max_amplicon_length);
	DDV_MinMaxInt(pDX, m_max_amplicon_length, 0, 999999);
	DDX_Text(pDX, IDC_MIN_AMPLICON_LEN_EDIT, m_min_amplicon_length);
	DDV_MinMaxInt(pDX, m_min_amplicon_length, 0, 999999);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CHybridDialog, CDialog)
	//{{AFX_MSG_MAP(CHybridDialog)
	ON_BN_CLICKED(IDC_CLOSE_BUTTON, OnCloseHybrid)
	ON_BN_CLICKED(ID_PRIMER_BUTTON, OnPrimerButton)
	ON_BN_CLICKED(IDC_NEXT_HYBRID_BUTTON, OnNextHybridButton)
	ON_BN_CLICKED(IDC_PROBE_BUTTON, OnProbeButton)
	ON_BN_CLICKED(IDC_ANNOTATE_HYBRID_BUTTON, OnAnnotateHybridButton)
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDCANCEL, OnCloseHybrid)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHybridDialog message handlers

BOOL CHybridDialog::Create() 
{
	return CDialog::Create(CHybridDialog::IDD);
}

BOOL CHybridDialog::OnInitDialog() 
{
	CDialog::OnInitDialog();

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


void CHybridDialog::OnCloseHybrid() 
{
	if(m_pView != NULL){
		m_pView->PostMessage(WM_HYBRID_CLOSE, 0);
	}
}

void CHybridDialog::OnPrimerButton() 
{
	// Only attempt to hybridize if all variables pass the
	// dialog's automatic validation step
	if( UpdateData(TRUE) && (m_pView != NULL) ){
		m_pView->PostMessage(WM_HYBRID_PRIMER, 0);
	}
}

void CHybridDialog::OnProbeButton() 
{
	// Only attempt to hybridize if all variables pass the
	// dialog's automatic validation step
	if( UpdateData(TRUE) && (m_pView != NULL) ){
		m_pView->PostMessage(WM_HYBRID_PROBE, 0);
	}
}

void CHybridDialog::OnNextHybridButton() 
{
	// Tell the view to display the next search results
	if(m_pView != NULL){
		m_pView->PostMessage(WM_HYBRID_NEXT, 0);
	}
}

void CHybridDialog::OnAnnotateHybridButton() 
{
	// Tell the view to annotation the currently selected sequence
	if(m_pView != NULL){
		m_pView->PostMessage(WM_HYBRID_ANNOTATE, 0);
	}
}
