// This program was prepared by the Regents of the University of California at 
// Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36 
// with the U.S. Department of Energy (DOE). All rights in the program are reserved 
// by the DOE and the University.  Permission is granted to the public to copy and 
// use this software without charge, provided that this Notice and any statement of 
// authorship are reproduced on all copies.  Neither the U.S. Government nor the 
// University makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this software.


#include "stdafx.h"
#include "GV.h"
//#include "GVView.h"
#include "GVDoc.h"
#include "OptionsDialogEx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COptionsDialogEx dialog


COptionsDialogEx::COptionsDialogEx(CView* pParent)
	: CDialog(COptionsDialogEx::IDD, pParent)
{
	m_pView = pParent;

	//{{AFX_DATA_INIT(COptionsDialogEx)
	m_font_value = _T("");
	m_gc_value = _T("");
	m_na_value = _T("");
	m_strand_value = _T("");
	m_tm_value = _T("");
	//}}AFX_DATA_INIT
}


void COptionsDialogEx::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(COptionsDialogEx)
	DDX_Control(pDX, IDC_SLIDER_TM, m_slider_tm);
	DDX_Control(pDX, IDC_SLIDER_STRAND, m_slider_strand);
	DDX_Control(pDX, IDC_SLIDER_NA, m_slider_na);
	DDX_Control(pDX, IDC_SLIDER_GC, m_slider_gc);
	DDX_Control(pDX, IDC_SLIDER_FONT, m_slider_font);
	DDX_Text(pDX, IDC_STATIC_FONT, m_font_value);
	DDX_Text(pDX, IDC_STATIC_GC, m_gc_value);
	DDX_Text(pDX, IDC_STATIC_NA, m_na_value);
	DDX_Text(pDX, IDC_STATIC_STRAND, m_strand_value);
	DDX_Text(pDX, IDC_STATIC_TM, m_tm_value);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(COptionsDialogEx, CDialog)
	//{{AFX_MSG_MAP(COptionsDialogEx)
	ON_BN_CLICKED(IDC_CLOSE_BUTTON, OnCloseButton)
	ON_WM_HSCROLL()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDCANCEL, OnClose)
	ON_MESSAGE(WM_OPTIONS_UPDATE, OnOptionsUpdate)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COptionsDialogEx message handlers

BOOL COptionsDialogEx::Create() 
{
	return CDialog::Create(COptionsDialogEx::IDD);
}

BOOL COptionsDialogEx::OnInitDialog() 
{
	CDialog::OnInitDialog();

	CGVDoc* pDoc = (CGVDoc*)(m_pView->GetDocument());
	ASSERT_VALID(pDoc);
	
	// Initialize the text display and slider controls
	m_font_value.Format("%f", pDoc->gp.font_scale());
	m_slider_font.SetRange(pDoc->gp.min_font_scale_index(), 
		pDoc->gp.max_font_scale_index());
	m_slider_font.SetPos(pDoc->gp.font_scale_index());

	m_gc_value.Format("%d", pDoc->gp.content_window());
	m_slider_gc.SetRange(pDoc->gp.min_content_window(), 
		pDoc->gp.max_content_window());
	m_slider_gc.SetPos(pDoc->gp.content_window());

	m_tm_value.Format("%d", pDoc->gp.tm_window());
	m_slider_tm.SetRange(pDoc->gp.min_tm_window(), 
		pDoc->gp.max_tm_window());
	m_slider_tm.SetPos(pDoc->gp.tm_window());

	m_na_value.Format("%.3g", pDoc->gp.tm_salt());
	m_slider_na.SetRange(pDoc->gp.min_tm_salt_index(), 
		pDoc->gp.max_tm_salt_index());
	m_slider_na.SetPos(pDoc->gp.tm_salt_index());

	m_strand_value.Format("%.3g", pDoc->gp.tm_strand());
	m_slider_strand.SetRange(pDoc->gp.min_tm_strand_index(), 
		pDoc->gp.max_tm_strand_index());
	m_slider_strand.SetPos(pDoc->gp.tm_strand_index());

	UpdateData(FALSE);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}


void COptionsDialogEx::OnCloseButton() 
{
	// Tell the view to close this dialog
	if(m_pView != NULL){
		m_pView->PostMessage(WM_OPTIONS_CLOSE, 0);
	}
}

void COptionsDialogEx::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
	
	if(!pScrollBar || !m_pView){
		CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
		return;
	}

	if((CSliderCtrl *)pScrollBar == &m_slider_font){
		// Update the font size
		m_pView->PostMessage(WM_OPTIONS_UPDATE, WM_OPTIONS_SLIDE_FONT, m_slider_font.GetPos());
	}

	if((CSliderCtrl *)pScrollBar == &m_slider_gc){
		// Update the font size
		m_pView->PostMessage(WM_OPTIONS_UPDATE, WM_OPTIONS_SLIDE_GC, m_slider_gc.GetPos());
	}

	if((CSliderCtrl *)pScrollBar == &m_slider_tm){
		// Update the font size
		m_pView->PostMessage(WM_OPTIONS_UPDATE, WM_OPTIONS_SLIDE_TM, m_slider_tm.GetPos());
	}

	if((CSliderCtrl *)pScrollBar == &m_slider_na){
		// Update the font size
		m_pView->PostMessage(WM_OPTIONS_UPDATE, WM_OPTIONS_SLIDE_NA, m_slider_na.GetPos());
	}

	if((CSliderCtrl *)pScrollBar == &m_slider_strand){
		// Update the font size
		m_pView->PostMessage(WM_OPTIONS_UPDATE, WM_OPTIONS_SLIDE_STRAND, m_slider_strand.GetPos());
	}

	CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
}

void COptionsDialogEx::OnClose() 
{
	// Tell the view to close this dialog
	if(m_pView != NULL){
		m_pView->PostMessage(WM_OPTIONS_CLOSE, 0);
	}
}

LRESULT COptionsDialogEx::OnOptionsUpdate(WPARAM wParam, LPARAM lParam)
{
	CGVDoc* pDoc = (CGVDoc*)(m_pView->GetDocument());
	ASSERT_VALID(pDoc);

	// What event occured/value needs to be updated?
	switch(wParam){
		case WM_OPTIONS_SLIDE_FONT:
			m_font_value.Format("%f", pDoc->gp.font_scale());
			break;
		case WM_OPTIONS_SLIDE_GC:
			m_gc_value.Format("%d", pDoc->gp.content_window());
			break;
		case WM_OPTIONS_SLIDE_TM:
			m_tm_value.Format("%d", pDoc->gp.tm_window());
			break;
		case WM_OPTIONS_SLIDE_NA:
			m_na_value.Format("%.3g", pDoc->gp.tm_salt());
			break;
		case WM_OPTIONS_SLIDE_STRAND:
			m_strand_value.Format("%.3g", pDoc->gp.tm_strand());
			break;
	};

	UpdateData(FALSE);

	return 0L;
}
