// This program was prepared by the Regents of the University of California at 
// Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36 
// with the U.S. Department of Energy (DOE). All rights in the program are reserved 
// by the DOE and the University.  Permission is granted to the public to copy and 
// use this software without charge, provided that this Notice and any statement of 
// authorship are reproduced on all copies.  Neither the U.S. Government nor the 
// University makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this software.

#include "stdafx.h"

// These are helper functions to deal periodic varibles

// Map a value to [0 ... m_period - 1]
//unsigned long wrap(const long &m_val, const unsigned int &m_period)
//{
//	return (m_val >= 0) ? m_val % m_period : m_period - (-m_val % m_period);
//}

#ifdef MOVED_ALL_FUNC_TO_DOT_H_FOR_INLINE
inline unsigned int wrap(const int &m_val, const unsigned int &m_period)
{
	return (m_val >= 0) ? m_val % m_period : m_period - (-m_val % m_period);
}


// Do two regions ({m_start_a, m_stop_a} and {m_start_b, m_stop_b}) 
// overlap on the periodic interval [0 ... m_period - 1]?
bool overlap(unsigned int m_start_a, unsigned int m_stop_a, 
			 unsigned int m_start_b, unsigned int m_stop_b,
			 const unsigned int &m_period)
{
	// Wrap all start/stop values
	m_start_a = wrap(m_start_a, m_period);
	m_stop_a = wrap(m_stop_a, m_period);

	m_start_b = wrap(m_start_b, m_period);
	m_stop_b = wrap(m_stop_b, m_period);

	if(m_start_a <= m_stop_a){
		if(m_start_b <= m_stop_b){
			if( (m_start_b > m_stop_a) || (m_stop_b < m_start_a) ){
				return false;
			}
		}
		else{ // m_start_b > m_stop_b
			
			if( (m_stop_a < m_start_b) && (m_start_a > m_stop_b) ){
				return false;
			}

		}	

	}
	else{ // m_start_a > m_stop_a
		if(m_start_b <= m_stop_b){
			if( (m_stop_b < m_start_a) && (m_start_b > m_stop_a) ){
				return false;
			}
		}
		// m_start_b > m_stop_b
		//else{ 
		//	Both segments wrap around and must overlap
		//	return true; 
		//}
	}

	return true;
}

// Does a point, m_a, overlap a region, {m_start_b, m_stop_b}, on the periodic interval
// [0 ... m_period - 1]?
bool overlap(unsigned int m_a,
			 unsigned int m_start_b, unsigned int m_stop_b,
			 const unsigned int &m_period)
{
	m_a = wrap(m_a, m_period);
	m_start_b = wrap(m_start_b, m_period);
	m_stop_b = wrap(m_stop_b, m_period);

	if(m_start_b <= m_stop_b){
		if((m_a < m_start_b) || (m_a > m_stop_b)){
			return false;
		}
	}
	else{ // m_start_b > m_stop_b
		if((m_a >= m_start_b) || (m_a <= m_stop_b)){
			return true;
		}

		return false;
	}

	return true;
}

// What is the center of the region {m_start_a, m_stop_a} on the periodic
// interval [0 ... m_period - 1]?
unsigned int center(const unsigned int &m_start, const unsigned int &m_stop,
					const unsigned int &m_period)
{

	return (m_start <= m_stop) ? (m_start + m_stop)/2 :
		wrap(m_start + (m_period - m_start + m_stop)/2, m_period);

}

#endif // MOVED_ALL_FUNC_TO_DOT_H_FOR_INLINE
