// This program was prepared by the Regents of the University of California at 
// Los Alamos National Laboratory (the University) under contract No. W-7405-ENG-36 
// with the U.S. Department of Energy (DOE). All rights in the program are reserved 
// by the DOE and the University.  Permission is granted to the public to copy and 
// use this software without charge, provided that this Notice and any statement of 
// authorship are reproduced on all copies.  Neither the U.S. Government nor the 
// University makes any warranty, express or implied, or assumes any liability or 
// responsibility for the use of this software.

#include "stdafx.h"
#include "univout.h"

#include <sstream>
#include <iostream>

using namespace std;

// By default, output to the console
#define	UNIVOUT_CONSOLE

// Platform dependant local functions and includes
#ifdef _WINDOWS

#undef UNIVOUT_CONSOLE

#include "MainFrm.h"
#include "GV.h"

void flush_win32(const string &m_buffer);
#endif // _WINDOWS

#ifdef _MOTIF

#undef UNIVOUT_CONSOLE

#include <Xm/Text.h>
 
void flush_motif(const string &m_buffer);
extern Widget status_bar_info;
#endif // _MOTIF

#ifdef _MAC_OSX

#undef UNIVOUT_CONSOLE

void flush_mac_osx(const string &m_buffer);
#endif // _MAC_OSX

#ifdef UNIVOUT_CONSOLE

void flush_console(const string &m_buffer);
#endif // UNIVOUT_CONSOLE


// Global variable
univout uout;

univout& univout::operator << (const univout_terminator &m_endl)
{
	
	flush();
	
	buffer = "";

	return (*this);
}

univout& univout::operator << (const univout &m_out)
{
	buffer += m_out.buffer;

	return (*this);
}

univout& univout::operator << (const std::string &m_text)
{
	buffer += m_text;

	return (*this);
}

univout& univout::operator << (const char &m_char)
{
	stringstream s;

	s << m_char;

	buffer += s.str();

	return (*this);
}

univout& univout::operator << (const int &m_int)
{

	stringstream s;

	s << m_int;

	buffer += s.str();

	return (*this);
}

univout& univout::operator << (const unsigned int &m_uint)
{

	stringstream s;

	s << m_uint;

	buffer += s.str();

	return (*this);
}

#ifdef _MAC_OSX
univout& univout::operator << (const size_t &m_size_t)
{

	stringstream s;

	s << m_size_t;

	buffer += s.str();

	return (*this);
}
#endif // _MAC_OSX

univout& univout::operator << (const double &m_double)
{

	stringstream s;

	s << m_double;

	buffer += s.str();

	return (*this);
}

univout& univout::operator << (const long &m_long)
{

	stringstream s;

	s << m_long;

	buffer += s.str();

	return (*this);
}

// The flush function is the platform dependant part of the code
void univout::flush()
{
#ifdef _WINDOWS
	flush_win32(buffer);
#endif // _WINDOWS

#ifdef _MOTIF
        flush_motif(buffer);
#endif // _MOTIF

#ifdef _MAC_OSX
    flush_mac_osx(buffer);
#endif // _MAC_OSX

#ifdef UNIVOUT_CONSOLE
	flush_console(buffer);
#endif // UNIVOUT_CONSOLE
}

#ifdef _MOTIF
void flush_motif(const string &m_buffer)
{
        XmString str = XmStringCreateLocalized((char*)(m_buffer.c_str()));
 
        XtVaSetValues(status_bar_info, XmNlabelString, str, NULL);
 
        XmStringFree(str);
}
#endif // _MOTIF


#ifdef _WINDOWS
void flush_win32(const string &m_buffer)
{
	CMainFrame *pFrame = (CMainFrame*)AfxGetApp()->m_pMainWnd;

	if(pFrame){
		CStatusBar* pStatus = &pFrame->m_wndStatusBar;

		pStatus->SetPaneText(INFO_PANE, m_buffer.c_str());
	}

}
#endif // _WINDOWS

#ifdef _MAC_OSX
void flush_mac_osx(const string &m_buffer)
{
	WindowRef window = FrontNonFloatingWindow ();
        
	if(window){
		CFStringRef title = CFStringCreateWithCString(NULL, m_buffer.c_str(), CFStringGetSystemEncoding());

		SetWindowTitleWithCFString(window, title);
                
		CFRelease(title);
	}

}
#endif // _MAC_OSX

#ifdef UNIVOUT_CONSOLE
void flush_console(const string &m_buffer)
{
	cerr << m_buffer << endl;
}

#endif // UNIVOUT_CONSOLE
